﻿#include "toptemplateclass1a.h"
#include <QtWidgets>
#include <tbaseutil/tconcurrent.h>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tdataresponse.h>
#include <tbaseutil/tenumlist.h>
#include <tbaseutil/ttheme.h>
#include <tbaseutil/tresource.h>
#include <tdatabaseutil/tsqlqueryv2.h>
#include <tdatabaseutil/tsqlselectorv2.h>
#include <tdatabaseutil/tsqlconnectionpoolv2.h>
#include <topcore/topcore.h>
#include <topcore/topclasssqlthread.h>
#include <topcore/topenummanager.h>
#include <twidget/tsplitter.h>
#include <twidget/ttreeview.h>
#include <twidget/tsearchentry.h>
#include <twidget/taccordion.h>
#include <twidget/tcategorytreeview.h>
#include <twidget/tuiloader.h>

/*!
 * \class TopTemplateClass1A
 * \inmodule templateclass
 * \brief 该模板支持导航栏和一个TreeView，支持搜索
 */

TopTemplateClass1A::TopTemplateClass1A(const QString &iModuleName,
                                       const QVariantMap &iUrlPars,
                                       QWidget *iParent)
    : TopClassAbs(iParent)
{
    initModule(iModuleName, iUrlPars);
    QString licenseKey = config("sys_license_key").toString();
    if (!licenseKey.isEmpty()) {
        setLicenseKey(licenseKey);
    }
    QStringList langLst = config("sys_lang_list").toStringList();
    for (const auto &lang : langLst) {
        appendLanguage(lang);
    }
    if (isHookExists("afterModuleInit")) {
        callHooks("afterModuleInit");
    }
    TopClassHelper::parseTableConf0(this, "view", &mTreeConf);
    initUi();
    if (isHookExists("afterViewInit")) {
        callHooks("afterViewInit");
    }
    restoreSizeState();
    restoreObjectState(mTreeView);
    QTimer::singleShot(0, this, SLOT(refresh()));
}

TopTemplateClass1A::~TopTemplateClass1A()
{
    saveSizeState();
    saveObjectState(mTreeView);
    if (isHookExists("onDestroy")) {
        callHooks("onDestroy");
    }
}

/*!
 * \brief 刷新数据
 */
void TopTemplateClass1A::refresh()
{
    if (mTreeConf.dbSql.isEmpty() && mTreeConf.dbTableName.isEmpty()) {
        return;
    }

    QString groupKey = config("view.group_key").toString();
    QString parentKey = config("view.parent_key").toString();
    QString childKey = config("view.child_key").toString();
    QString seqKey = config("view.seq_key").toString();

    TSqlSelectorV2 selector;
    // 导航栏过滤项
    if (mNaviType == NaviType::None) {
        // do nothing
    } else if (mNaviType == NaviType::UiLoader) {
        QString whereStr = uiloaderWhere();
        if (!whereStr.isEmpty()) {
            selector.addWhere(whereStr);
        }
    } else {
        if (!mCategoryWhere.isEmpty()) {
            selector.addWhere(mCategoryWhere.toSql());
        }
    }
    if (mTreeConf.dbSql.isEmpty()) {
        selector.setTable(mTreeConf.dbTableName);
        QStringList fields = mTreeConf.queryFields;
        fields.removeOne("_category");
        selector.setField(fields);
        selector.setReturnRowCount(false);

        loading(ttr("Loading"));
        QVariant data = doThreadWork(new TopClassSqlThread(this),
                                     TOPSQLTHREAD_SELECT_ARRAYMAP,
                                     QVariant::fromValue(selector));
        unloading();
        TDataResponse response(data.toMap());
        if (response.hasError()) {
            alertError(ttr("Load data failed!"));
        } else {
            QVariantList tableData = response.data().toList();
            if (!groupKey.isEmpty()) {
                loadDataByGrouping(tableData, groupKey, seqKey);
            } else if (!parentKey.isEmpty() && !childKey.isEmpty()) {
                loadDataByParent(tableData, parentKey, childKey, seqKey);
            } else {
                Q_ASSERT(false);
            }
            alertOk(ttr("Data loaded"));
        }
    } else {
        selector.setTable(QString("(%1) _TEMP_TABLE_").arg(mTreeConf.dbSql));
        selector.setField(mTreeConf.queryFields);
        loading(ttr("Loading"));
        QVariant tableData = doThreadWork(new TopClassSqlThread(this),
                                     TOPSQLTHREAD_SELECT_ARRAYMAP,
                                     QVariant::fromValue(selector));
        unloading();
        TDataResponse response(tableData.toMap());
        if (response.hasError()) {
            alertError(ttr("Load data failed!"));
            return;
        }
        loadTableData(response.data().toList(), groupKey, parentKey, seqKey);
        alertOk(ttr("Data loaded"));
    }
}

/*!
 * \brief 树形控件对象
 */
TTreeView *TopTemplateClass1A::treeView() const
{
    return mTreeView;
}

/*!
 * \brief 根据配置生成的查询数据库的字段
 */
QStringList TopTemplateClass1A::queryFields() const
{
    return mTreeConf.queryFields;
}

/*!
 * \brief 当前选中条目的primaryKey
 */
QVariantList TopTemplateClass1A::selectedItems() const
{
    return mSelectedLst;
}

/*!
 * \brief 当前选中条目的所有数据
 */
QVariantList TopTemplateClass1A::selectedDataMaps() const
{
    return mTreeView->selectedRowDataMaps(true);
}

/*!
 * \brief 导入树形结构的数据
 * [
 *   {
 *     "name": "",
 *     "CHILDREN": [
 *       {
 *         "name": ""
 *       }
 *     ]
 *   }
 * ]
 */
void TopTemplateClass1A::loadTreeData(const QVariantList &iTreeData,
                                      const QString &iChildKey)
{
    mTreeView->loadTreeData(iTreeData, iChildKey);
}

/*!
 * \brief 导入列表数据，此接口已废弃
 * \param iGroupKey 分组Key
 * \param iParentKey 父结点Key, 此值为空时只按iGroupKey分组，树形只有一级
 * \param iSeqKey 同级结点排序的Key
 */
void TopTemplateClass1A::loadTableData(const QVariantList &iTableData,
                                       const QString &iGroupKey,
                                       const QString &iParentKey,
                                       const QString &iSeqKey)
{
    Q_UNUSED(iSeqKey)

    QVariantList tableData = iTableData;
    TopClassHelper::formatTableData(this, &mTreeConf, tableData);
    QVariantList data;
    if (iParentKey.isEmpty()) {
        QVariantMap categoryMap;
        for (const auto & i : tableData) {
            QVariantMap iMap = i.toMap();
            QString category = iMap.value(iGroupKey).toString();
            QVariantList items = categoryMap.value(category).toList();
            items.append(i);
            categoryMap.insert(category, items);
        }
        QMapIterator<QString, QVariant> iter(categoryMap);
        while (iter.hasNext()) {
            iter.next();
            QVariantMap item;
            item.insert("_category", iter.key());
            item.insert("CHILDREN", iter.value());
            data.append(item);
        }
    } else {
        // 此接口已废弃，不支持此分支
        Q_ASSERT(false);
    }
    if (isHookExists("handleTableData")) {
        data = this->callHooksQuick("handleTableData", QVariantList()<<QVariant(data)).toVariant().toList();
    }
    mTreeView->loadTreeData(data);
}

/*!
 * \brief 导入列表数据，根据 \a iGroupKey  进行分组，此树形结构只有一级
 */
void TopTemplateClass1A::loadDataByGrouping(const QVariantList &iTableData,
                                            const QString &iGroupKey,
                                            const QString &iSeqKey)
{
    Q_UNUSED(iSeqKey)
    QVariantList tableData = iTableData;
    TopClassHelper::formatTableData(this, &mTreeConf, tableData);
    QVariantList data;
    QVariantMap categoryMap;
    for (const auto & i : tableData) {
        QVariantMap iMap = i.toMap();
        QString category = iMap.value(iGroupKey).toString();
        QVariantList items = categoryMap.value(category).toList();
        items.append(i);
        categoryMap.insert(category, items);
    }
    QMapIterator<QString, QVariant> iter(categoryMap);
    while (iter.hasNext()) {
        iter.next();
        QVariantMap item;
        item.insert("_category", iter.key());
        item.insert("CHILDREN", iter.value());
        data.append(item);
    }
    if (isHookExists("handleTableData")) {
        data = this->callHooksQuick("handleTableData", QVariantList()<<QVariant(data)).toVariant().toList();
    }
    mTreeView->loadTreeData(data);
}

/*!
 * \brief 导入列表数据，根据 \a iParentKey 和 \a iChildKey 生成树，此树形结构可能有多级
 */
void TopTemplateClass1A::loadDataByParent(const QVariantList &iTableData,
                                          const QString &iParentKey,
                                          const QString &iChildKey,
                                          const QString &iSeqKey)
{
    QVariantList tableData = iTableData;
    TopClassHelper::formatTableData(this, &mTreeConf, tableData);
    if (isHookExists("handleTableData")) {
        tableData = this->callHooksQuick("handleTableData", QVariantList()<<QVariant(tableData)).toVariant().toList();
    }
    mTreeView->loadTreeData(buildTree(tableData, iParentKey, iChildKey, iSeqKey));
}

/*!
 * \brief 当选中项发生改变时的槽函数
 */
void TopTemplateClass1A::onSelectionChanged()
{
    mSelectedLst = mTreeView->selectedPrimaryKeys();
    refreshActionState();
}

void TopTemplateClass1A::onCategoryViewDataChanged(const QVariantList iDataLst)
{
    mCategoryWhere.clear();
    QString format = config("navi.format", "filter_by_enum").toString();
    if (!iDataLst.isEmpty()) {
        if (format.compare("filter_by_enum") == 0) {
            QVariantMap cateMap;
            for (QVariant item: iDataLst) {
                QVariantMap itemMap = item.toMap();
                QString data = itemMap.value("data").toString();
                QString name = itemMap.value("name").toString();
                if (data.isEmpty()) {
                    continue;
                }
                cateMap.insertMulti(data,name);
            }
            for (QString field: cateMap.uniqueKeys()) {
                QString dbField = field;
                TSqlWhereCompsiteV2 childWhere;
                childWhere.setLogic(TSqlWhereCompsiteV2::Logic_Or);
                for (QVariant ele: cateMap.values(field)) {
                    QString value = ele.toString();
                    if (mTreeConf.fieldFormatMap.contains(dbField)) {
                        QString fieldValue = mTreeConf.fieldFormatMap.value(dbField).toString();
                        if (fieldValue.compare("array") == 0) {
                            QString sqlStr = QString("%1 @> '{%2}' ").arg(dbField).arg(value);
                            childWhere.append(sqlStr);
                        }
                    } else {
                        childWhere.append(dbField, value);
                    }
                }
                mCategoryWhere.append(childWhere);
            }
        } else if (format.compare("filter_by_sql_where") == 0) {
            QVariantMap item = iDataLst.at(0).toMap();
            QString sqlWhere = item.value("data").toString();
            if (!sqlWhere.isEmpty()) {
                mCategoryWhere.append(sqlWhere);
            }
        }
    }
    refresh();
}

/*!
 * \brief 搜索
 */
void TopTemplateClass1A::onSearch(const QString &iText, const QVariant &iFields)
{
    QRegExp regex(iText);
    regex.setPatternSyntax(QRegExp::Wildcard);
    regex.setCaseSensitivity(Qt::CaseInsensitive);
    auto searchCols = iFields.toStringList();
    if (searchCols.isEmpty()) {
        searchCols = mSearchCols;
    }
    mTreeView->findRows(regex, searchCols);
}

void TopTemplateClass1A::onNaviReset()
{
    if (TUiLoader *filterUi = qobject_cast<TUiLoader*>(mNaviView)) {
        return;
    } else {
        filterUi->clearValues();
        this->refresh();
    }
}

TopTemplateClass1A::NaviType TopTemplateClass1A::str2NaviType(const QString &iType)
{
    const QString type = iType.toUpper();
    if (type == "UILOADER") {
        return NaviType::UiLoader;
    } else if (type == "NONE") {
        return NaviType::None;
    } else {
        return NaviType::CategoryTreeView;
    }
}

QVariantList TopTemplateClass1A::parseCategoryConfig()
{
    QVariantList categoryLst;

    QVariantList categoryConfig = config("navi.categories").toList();
    QString format = config("navi.format", "filter_by_enum").toString();
    if (format.compare("filter_by_enum") == 0) {
        for(const QVariant &item: categoryConfig) {
            QVariantMap itemMap = item.toMap();
            QString itemName         = itemMap.value("name").toString();
            QVariantList childrenLst = itemMap.value("children").toList();
            QString enumCfg          = itemMap.value("enum_children").toString();
            QStringList enumCheckedLst = itemMap.value("enum_checked").toStringList();
            QStringList enumInvisibleLst = itemMap.value("enum_invisible").toStringList();
            if (itemMap.contains("visible")) {
                itemMap.insert("VISIBLE", itemMap.take("visible"));
            }
            if (itemMap.contains("expand")) {
                itemMap.insert("EXPAND", itemMap.take("expand"));
            }
            TSqlWhereCompsiteV2 childWhere;
            childWhere.setLogic(TSqlWhereCompsiteV2::Logic_Or);
            QVariantList type = TOPENM->enumList(enumCfg)->toComboList();
            for (const QVariant &item: type) {
                QVariantMap enumMap = item.toMap();
                QString enumName = enumMap.value("name").toString();
                QVariantMap childMap;
                childMap.insert("name",enumName);
                childMap.insert("text",ttr(enumMap.value("text").toString()));
                childMap.insert("icon",enumMap.value("icon"));
                childMap.insert("data", itemName);
                if(enumCheckedLst.contains(enumName)) {
                    childMap.insert("checked",1);
                } else {
                    childMap.insert("checked",0);
                }
                if (enumInvisibleLst.contains(enumName)) {
                    childMap.insert("VISIBLE", 0);
                } else {
                    childMap.insert("VISIBLE",1);
                }
                childrenLst.append(childMap);
            }
            int checkedChildLen = 0;
            for (QVariant &item : childrenLst) {
                QVariantMap itemMap = item.toMap();
                QString name = itemMap.value("name").toString();
                if(itemMap.value("checked").toInt() == 1) {
                    childWhere.append(itemName, name);
                    checkedChildLen++;
                }
                item = itemMap;
            }
            if (checkedChildLen == childrenLst.length()) {
                itemMap.insert("checked", 1);
            } else {
                itemMap.insert("checked", 0);
            }
            mCategoryWhere.append(childWhere);

            itemMap.remove("children");
            itemMap.remove("enum_children");
            itemMap.remove("enum_checked");
            itemMap.remove("enum_invisible");
            itemMap.insert("text",ttr(itemMap.value("text").toString()));
            itemMap.insert("CHILDREN",childrenLst);

            categoryLst.append(itemMap);
        }
    } else if (format.compare("filter_by_sql_where") == 0) {
        return categoryConfig;
    }
    return categoryLst;
}

QString TopTemplateClass1A::uiloaderWhere() const
{
    TUiLoader *filterUi = filterUi = qobject_cast<TUiLoader*>(mNaviView);
    if (filterUi == nullptr) {
        return QString();
    }

    auto filterDataMap = filterUi->getAllValues().toVariant().toMap();

    QString searchSql;
    QMapIterator<QString, QVariant> iter(filterDataMap);
    while (iter.hasNext()) {
        iter.next();
        QString key = iter.key();
        QString value = iter.value().toString();
        if (value.isEmpty()) {
            continue;
        }
        auto obj = filterUi->getObject(key);
        if (obj != nullptr) {
            QVariantMap userData = obj->property("tui_user_data").toMap();
            if (userData.contains("field_name")) {
                key = userData.value("field_name").toString();
            }
            QString operatorStr = "=";
            if (userData.contains("operator")) {
                operatorStr = userData.value("operator").toString();
            }
            if (operatorStr.compare("like", Qt::CaseInsensitive) == 0 ||
                    operatorStr.compare("ilike", Qt::CaseInsensitive) == 0) {
                value = QString("'%%1%'").arg(value);
            } else if (operatorStr.compare("in", Qt::CaseInsensitive) == 0) {
                value = QString("(%1)").arg(value);
            } else {
                if (iter.value().type() == QVariant::String) {
                    value = QString("'%1'").arg(value);
                }
            }
            QString searchStr = QString("%1 %2 %3").arg(key).arg(operatorStr).arg(value);
            if (!searchSql.isEmpty()) {
                searchSql.append(" and ");
            }
            searchSql.append(searchStr);
        }
    }
    return searchSql;
}

/*!
 * \brief 初始化界面
 */
void TopTemplateClass1A::initUi()
{
    mBodySplitter = new TSplitter(this);
    mHSplitter = new TSplitter(this);
    setCentralWidget(mBodySplitter);
    mBodySplitter->addWidget(mHSplitter);

    //导航栏
    initNavi();

    //树形表
    mBodyWgt = new QWidget(this);
    QVBoxLayout *bodyLayout = new QVBoxLayout(mBodyWgt);
    bodyLayout->setMargin(0);
    bodyLayout->setSpacing(0);

    mTreeView = new TTreeView(this);
    mTreeView->setObjectName("mTreeView");
    mTreeView->header()->setStretchLastSection(true);
    connect(mTreeView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SLOT(onSelectionChanged()));
    QStringList dataKeys = mTreeConf.dataKeys;
    dataKeys.append("_category");
    mTreeView->setDataKeyList(dataKeys);
    mTreeView->setPrimaryKey(mTreeConf.primaryKey);
    mTreeView->setHeaderItem(mTreeConf.horizontalHeaders);
    mTreeView->setSelectionMode(QAbstractItemView::ExtendedSelection);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("MAIN_TOOLBAR"))) {
        bodyLayout->addWidget(toolbar, 0);
        mSearchEntry = qobject_cast<TSearchEntry *>(uim()->getWidget("MAIN_TOOLBAR/SEARCH_ENTRY"));
        if (mSearchEntry != nullptr) {
            auto searchLst = TDataParse::headerItem2searchList(mTreeView->headerItem());
            for (auto i : searchLst) {
                mSearchCols.push_back(i.toMap().value("name").toString());
            }
            mSearchEntry->setOptionList(searchLst);
            mSearchEntry->setPlaceholderText(ttr("Search %1"));
            connect(mSearchEntry, SIGNAL(search(QString,QVariant)),
                    this, SLOT(onSearch(QString,QVariant)));
        }
    }
    if (QMenu *tablePopup = qobject_cast<QMenu *>(uim()->getWidget("TREEVIEW_POPUP"))) {
        mTreeView->setContextMenu(tablePopup);
    }

    bodyLayout->addWidget(mTreeView, 1);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("BOTTOM_TOOLBAR"))) {
        bodyLayout->addWidget(toolbar, 0);
    }

    mHSplitter->addWidget(mBodyWgt);
}

void TopTemplateClass1A::initNavi()
{
    mNaviType = str2NaviType(config("navi.__type__", "None").toString());
    if (mNaviType == NaviType::None) {
        return;
    }

    TAccordion *accordion = new TAccordion(this);
    accordion->setObjectName("__navi__");
    accordion->setProperty("SS_BG","NAVI");
    accordion->setMinimumWidth(TTHEME_DP(config("navi.min_width", 150).toInt()));
    accordion->setMaximumWidth(TTHEME_DP(config("navi.max_width", 300).toInt()));
    accordion->setAccordionLocation(TAccordion::AccordionLocation::Left);
    QVBoxLayout *naviLayout = new QVBoxLayout(accordion);
    naviLayout->setMargin(0);
    naviLayout->setSpacing(0);
    QLabel *naviLabel = new QLabel(tr("Navigation"));
    naviLabel->setProperty("SS_TYPE", "SUBHEADER");
    naviLayout->addWidget(naviLabel,0);
    naviLayout->addSpacing(TTHEME_DP(4));
    accordion->expandButton()->setFixedHeight(TTHEME_DP(40));
    accordion->expandButton()->setToolTip(tr("Show Navigation"));
    accordion->collapseButton()->setToolTip(tr("Hide Navigation"));
    accordion->setIsExpanded(config("navi.is_expanded", false).toBool());
    mHSplitter->addWidget(accordion);

    if (mNaviType == NaviType::UiLoader) {
        mNaviView = new TUiLoader(this);
        TUiLoader *filterUi = qobject_cast<TUiLoader*>(mNaviView);
        filterUi->setSelf(this);
        filterUi->setScriptEngine(APP->scriptEngine());
        filterUi->setUiStr(ui("navi").toString());

        QFrame *naviWgt = new QFrame(this);
        QVBoxLayout *leftLayout = new QVBoxLayout(naviWgt);
        leftLayout->addWidget(mNaviView);
        QHBoxLayout *leftBtnsLayout = new QHBoxLayout();
        leftBtnsLayout->setMargin(0);
        leftBtnsLayout->setSpacing(10);
        leftLayout->addLayout(leftBtnsLayout);
        QPushButton *resetBtn = new QPushButton(this);
        connect(resetBtn, SIGNAL(clicked(bool)), this, SLOT(onNaviReset()));
        leftBtnsLayout->addWidget(resetBtn);
        resetBtn->setText(tr("Reset"));
        resetBtn->setIcon(TRES->icon("rotate-right"));
        QPushButton *filterBtn = new QPushButton(this);
        connect(filterBtn, SIGNAL(clicked(bool)), this, SLOT(refresh()));
        leftBtnsLayout->addWidget(filterBtn);
        filterBtn->setText(tr("Filter"));
        filterBtn->setIcon(TRES->icon("filter"));
        leftLayout->addStretch(1);
        naviLayout->addWidget(naviWgt);
    } else {
        mNaviView = new TCategoryTreeView(this);
        mNaviView->setStyleSheet("TCategoryTreeView{border:NONE;background-color:transparent}");
        QVariantList rootChildList = parseCategoryConfig();
        if (rootChildList.isEmpty()) {
            mNaviView->setVisible(false);
            return;
        }
        TCategoryTreeView *categoryTreeView = qobject_cast<TCategoryTreeView *>(mNaviView);
        categoryTreeView->setItemList(rootChildList);
        categoryTreeView->setCheckable(config("navi.is_checkable").toBool());
        connect(categoryTreeView, SIGNAL(selectionChanged(QVariantList)),
                this, SLOT(onCategoryViewDataChanged(QVariantList)));
        naviLayout->addWidget(mNaviView);
    }
}

/*!
 * \brief 根据列表数据生成树形数据
 */
QVariantList TopTemplateClass1A::buildTree(const QVariantList &iDataLst,
                                           const QString &iParentKey,
                                           const QString &iChildKey,
                                           const QString &iSeqKey)
{
    if (iDataLst.isEmpty()) {
        return QVariantList();
    }

    QVariantList ret;
    QVariantList toplevel;
    QVariantMap parentDataMap;
    QVariantMap tmpDataMap;

    auto sequencePred = [iSeqKey](const QVariant &left, const QVariant &right) ->bool {
        return left.toMap().value(iSeqKey).toInt() < right.toMap().value(iSeqKey).toInt();
    };
    std::function<QVariantList(const QVariantMap &)> getChildren;
    getChildren = [&getChildren, &parentDataMap, &iChildKey, &sequencePred](const QVariantMap &iRowMap) ->QVariantList  {
        if (parentDataMap.contains(iRowMap.value(iChildKey).toString())) {
            QVariantList childLst;
            QVariantList tmp = parentDataMap.value(iRowMap.value(iChildKey).toString()).toList();
            for (const auto & child : tmp) {
                QVariantMap childMap = child.toMap();
                QVariantList children  = getChildren(childMap);
                if (!children.isEmpty()) {
                    std::sort(children.begin(), children.end(), sequencePred);
                    childMap.insert("CHILDREN", children);
                }
                childLst.append(childMap);
            }
            return childLst;
        } else {
            return QVariantList();
        }
    };

    for (const auto & i : iDataLst) {
        QVariantMap iMap = i.toMap();
        QVariantList childrenList = parentDataMap.value(
                    iMap.value(iParentKey).toString()).toList();
        childrenList.append(iMap);
        parentDataMap.insert(iMap.value(iParentKey).toString(), childrenList);
        tmpDataMap.insert(iMap.value(iChildKey).toString(), iMap);
    }

    for (const auto & i : iDataLst) {
        QVariantMap dataMap = i.toMap();
        QString parentId = dataMap.value(iParentKey).toString();
        if (!tmpDataMap.contains(parentId)) {
            toplevel.append(dataMap);
        }
    }

    for (const auto & i : toplevel) {
        QVariantMap iMap = i.toMap();
        QVariantList children = getChildren(iMap);
        if (!children.isEmpty()) {
            std::sort(children.begin(), children.end(), sequencePred);
            iMap.insert("CHILDREN", children);
        }
        ret.append(iMap);
    }
    std::sort(ret.begin(), ret.end(), sequencePred);
    return ret;
}
