﻿#include "toptemplateclass5.h"
#include <QtWidgets>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tdataresponse.h>
#include <tbaseutil/tenumlist.h>
#include <tbaseutil/tfileio.h>
#include <tbaseutil/ttheme.h>
#include <tdatabaseutil/tsqlqueryv2.h>
#include <tdatabaseutil/tsqlselectorv2.h>
#include <tdatabaseutil/tsqlconnectionpoolv2.h>
#include <topcore/topcore.h>
#include <tbaseutil/tresource.h>
#include <topcore/topclasssqlthread.h>
#include <topcore/topenummanager.h>
#include <twidget/taccordion.h>
#include <twidget/tadvancedquery.h>
#include <twidget/tcategorytreeview.h>
#include <twidget/tmessagebox.h>
#include <twidget/tpagetool.h>
#include <twidget/tscrollarea.h>
#include <twidget/tsearchentry.h>
#include <twidget/tsplitter.h>
#include <twidget/ttableview.h>
#include <twidget/ttabwidget.h>
#include <twidget/ttablechooserdialog.h>
#include <twidget/tuiloader.h>
#include <twidget/tuiloaderdialog.h>
#include <QRegularExpression>
namespace {
    QString nameWidthTablePrefix(const QString &iPrefix, const QString &iOldName) {
        QRegularExpression reg{QString(R"(table\(%1\))").arg(iPrefix)};
        QRegularExpressionMatch match = reg.match(iOldName);
        if(match.hasMatch()) {
            const QString matchStr = match.captured();
            return iOldName;
        } else {
            return QString("table(%1).%2").arg(iPrefix).arg(iOldName);
        }
    }

    void getPageToolData(TPageTool *iPageTool, const bool iResetPage, int &pageNum, int &pageSize) {
        if(iPageTool) {
            pageNum = 1;
            pageSize = -1;
            if(iResetPage) {
                iPageTool->setCurrentPage(1,true);
            }
            pageNum = iPageTool->currentPage();
            pageSize = iPageTool->pageSize();
        }
        if(pageNum < 1) { pageNum = 1; }
        if(pageSize < 1) { pageSize = 100; }
    }
}

/*!
 * \class TopTemplateClass5
 * \inmodule templateclass
 * \brief 带导航过滤和详情表单的主从表模板，导航栏只针对主表进行筛选。
 * navi有六种类型：
 * navi.__type__: None                              无导航，这是默认值
 * navi.__type__: CategoryTreeView                  导航树
 * navi.__type__: UiLoader                          定制的过滤界面，需配置navi.ui.js
 * navi.__type__: AdvancedQuery                     高级查询（暂不支持）
 * navi.__type__: NaviFilter                        使用TNaviFilter控件的导航（暂不支持）
 * navi.__type__: CategoryTreeViewAndAdvancedQuery  导航树加高级查询
 * navi.__type__: UiLoaderAndCategoryTreeView       定制的过滤界面加导航树
 */

TopTemplateClass5::TopTemplateClass5(const QString &iModuleName,
                                     const QVariantMap &iUrlPars,
                                     QWidget *iParent)
    : TopClassAbs(iParent)
{
    initModule(iModuleName, iUrlPars);
    QString licenseKey = config("sys_license_key").toString();
    if (!licenseKey.isEmpty()) {
        setLicenseKey(licenseKey);
    }
    QStringList langLst = config("sys_lang_list").toStringList();
    for (const auto &lang : langLst) {
        appendLanguage(lang);
    }
    if (isHookExists("afterModuleInit")) {
        callHooks("afterModuleInit");
    }
    TopClassHelper::parseTableConf0(this, "master_view", &mMasterTableConf);
    TopClassHelper::parseTableConf0(this, "slave_view", &mSlaveTableConf);
    initUi();
    connect(this, SIGNAL(isDetailModifiedChanged(bool)),
            this, SLOT(onDetailChanged(bool)));
    if (isHookExists("afterViewInit")) {
        callHooks("afterViewInit");
    }
//    restoreSizeState();
//    restoreObjectState(mLeftSplitter);
//    restoreObjectState(mBodySplitter);
//    restoreObjectState(mMasterView);
//    restoreObjectState(mSlaveView);
    refreshActionState();
    QTimer::singleShot(0, this, SLOT(refreshMaster()));
}

TopTemplateClass5::~TopTemplateClass5()
{
    saveSizeState();
    saveObjectState(mLeftSplitter);
    saveObjectState(mBodySplitter);
    saveObjectState(mMasterView);
    saveObjectState(mSlaveView);
    if (isHookExists("onDestroy")) {
        callHooks("onDestroy");
    }
}

/*!
 * \brief 刷新主表的数据，\a iResetPage 为true时表示重置分页
 */
void TopTemplateClass5::refreshMaster(bool iResetPage)
{
    mMasterView->clearSelection();
    QString sql = config("master_view.data_set.db_sql").toString();
    if (sql.isEmpty()) {
        if (!mMasterTableConf.dbTableName.isEmpty()) {
            TSqlSelectorV2 selector;
            selector.setTable(mMasterTableConf.dbTableName);
            selector.setField(mMasterTableConf.queryFields);
            selector.setReturnRowCount(true);
            // 搜索分页
            TopClassHelper::handleSearchPageOnQuery(mMasterSearchEntry, mMasterPageTool, iResetPage, &selector);
            // 排序
            TopClassHelper::handleOrderOnQuery(mMasterView, &mMasterTableConf, &selector);
            // 导航栏查询条件
            QString naviSql = naviWhere();
            if (!naviSql.isEmpty()) {
                selector.addWhere(naviSql);
            }
            loading(ttr("Loading"));
            QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_ARRAYMAP, QVariant::fromValue(selector));
            unloading();
            TDataResponse response(data.toMap());
            if (response.hasError()) {
                alertError(ttr("Load data failed!"));
            } else {
                if (mMasterPageTool != nullptr) {
                    mMasterPageTool->setRowCount(response.dataCount(), true);
                }
                QVariantList tableData = response.data().toList();
                TopClassHelper::formatTableData(this, &mMasterTableConf, tableData);
                mMasterView->loadData(tableData);
                alertOk(ttr("Data loaded"));
            }
        } else {
            // sql语句为空，dbname也为空时执行钩子获取master数据
            if (isHookExists("masterWillUpdate")) {
                int pageNum = 1;
                int pageSize = -1;
                QVariantMap orderInfo = TopClassHelper::handleOrderInfo(mMasterView, &mMasterTableConf);
                getPageToolData(mMasterPageTool,iResetPage,pageNum,pageSize);
                QScriptValue argVal = APP->scriptEngine()->newObject();
                argVal.setProperty("resetPage", iResetPage);
                argVal.setProperty("pageNum", pageNum);
                argVal.setProperty("pageSize", pageSize);
                argVal.setProperty("orderField", orderInfo.value("orderField").toString());
                argVal.setProperty("sortOrder", orderInfo.value("sortOrder").value<Qt::SortOrder>());
                QString naviSql = naviWhere();
                argVal.setProperty("naviWhere", naviSql);
                callHooks("masterWillUpdate", QScriptValueList() << argVal);
            }
            TopClassHelper::formatTableData(this, &mMasterTableConf, mMasterData);
            mMasterView->loadData(mMasterData);
            alertOk(ttr("Data loaded"));
        }
    } else {
        // @todo
    }
}

/*!
 * \brief 刷新从表的数据，\a iResetPage 为true时表示重置分页
 */
void TopTemplateClass5::refreshSlave(bool iResetPage)
{
    if(selectedMasterItems().size() == 0) {
        mSlaveView->loadData(QVariantList());
        if (mUiLoader) {
            mUiLoader->loadValues(QVariant());
        }
        return;
    }

    QString sql = config("slave_view.data_set.db_sql").toString();
    if (sql.isEmpty()) {
        if (!mSlaveTableConf.dbTableName.isEmpty()) {
            QString relationTable = config("master_view.relation.through").toString();
            QString slaveKey = config("master_view.relation.slave_key").toString();
            QString slaveForeignKey = config("master_view.relation.slave_foreign_key").toString();
            QString masterForeignKey = config("master_view.relation.master_foreign_key").toString();
            QStringList fields = mSlaveTableConf.queryFields;
            for (auto & f : fields) {
                f = QString("S.%1 AS %1").arg(f);
            }
            TSqlSelectorV2 selector;
            QString table = QString("%1 AS S LEFT JOIN %2 AS R ON S.%3 = R.%4")
                    .arg(mSlaveTableConf.dbTableName)
                    .arg(relationTable)
                    .arg(slaveKey)
                    .arg(slaveForeignKey);
            selector.setTable(table);
            selector.setField(fields);
            selector.setWhere("R."+masterForeignKey, mSelectedMasterItems.first());
            selector.setReturnRowCount(iResetPage);
            // 搜索分页
            TopClassHelper::handleSearchPageOnQuery(mSlaveSearchEntry, mSlavePageTool, iResetPage, &selector);
            // 排序
            TopClassHelper::handleOrderOnQuery(mSlaveView, &mSlaveTableConf, &selector);
            loading(ttr("Loading"));
            QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_ARRAYMAP, QVariant::fromValue(selector));
            unloading();
            TDataResponse response(data.toMap());
            if (response.hasError()) {
                alertError(ttr("Load data failed!"));
            } else {
                if (mSlavePageTool != nullptr && iResetPage) {
                    mSlavePageTool->setRowCount(response.dataCount(), true);
                }
                QVariantList tableData = response.data().toList();
                TopClassHelper::formatTableData(this, &mSlaveTableConf, tableData);
                mSlaveView->loadData(tableData);
                alertOk(ttr("Data loaded"));
            }
        } else {
            // sql语句为空，dbname也为空时执行钩子获取slave数据
            QString selectedPrimaryKey = mSelectedMasterItems.first().toString();
            if (isHookExists("slaveWillUpdate")) {
                QVariantMap argVal;
                if (mSlavePageTool != nullptr ) {
                    int pageNum = 1;
                    int pageSize = -1;
                    getPageToolData(mSlavePageTool, iResetPage, pageNum, pageSize);
                    argVal.insert("resetPage", iResetPage);
                    argVal.insert("pageNum", pageNum);
                    argVal.insert("pageSize", pageSize);
                }
                argVal = TDataParse::mergeVariantMap(argVal, TopClassHelper::handleOrderInfo(mSlaveView, &mSlaveTableConf));
                callHooksQuick("slaveWillUpdate", QVariantList() << selectedPrimaryKey<< argVal);
            }
            if (mSlavePageTool != nullptr && iResetPage) {
                mSlavePageTool->setRowCount(mMasterSlaveCount.value(selectedPrimaryKey), true);
            }
            QVariantList tableData = mMasterSlaveDataMap.value(selectedPrimaryKey).toList();
            TopClassHelper::formatTableData(this, &mSlaveTableConf, tableData);
            mSlaveView->loadData(tableData);
            alertOk(ttr("Data loaded"));
        }
    } else {
        // @todo
    }

    refreshDetail();
}

void TopTemplateClass5::refreshDetail()
{
    if (mUiLoader != nullptr) {
        QVariantMap dataMap;
        const QVariantMap masterMap = selectedMasterDataMaps().value(0).toMap();
        for(const QString &key: masterMap.keys()) {
            const QString tKey = ::nameWidthTablePrefix("master",key);
            dataMap.insert(tKey,masterMap[key]);
        }

        QVariantMap slaveMap = selectedSlaveDataMaps().value(0).toMap();
        for(const QString &key: slaveMap.keys()) {
            const QString tKey = ::nameWidthTablePrefix("slave",key);
            dataMap.insert(tKey,slaveMap[key]);
        }

        mUiLoader->loadValues(QVariant(dataMap));
        mUiLoader->refreshState();
    }
    setIsDetailModified(false);
}

/*!
 * \brief 主表格控件对象
 */
TTableView *TopTemplateClass5::masterView() const
{
    return mMasterView;
}

/*!
 * \brief 从表格控件对象
 */
TTableView *TopTemplateClass5::slaveView() const
{
    return mSlaveView;
}

const QStringList TopTemplateClass5::masterQueryFields() const
{
    return mMasterTableConf.queryFields;
}

const QStringList TopTemplateClass5::slaveQueryFields() const
{
    return mSlaveTableConf.queryFields;
}

/*!
 * \brief 主表当前选中条目的primaryKey
 */
QVariantList TopTemplateClass5::selectedMasterItems() const
{
    return mSelectedMasterItems;
}

/*!
 * \brief 主表当前选中条目的所有数据
 */
QVariantList TopTemplateClass5::selectedMasterDataMaps() const
{
    return mMasterView->selectedRowDataMaps(true);
}

/*!
 * \brief 从表当前选中条目的primaryKey
 */
QVariantList TopTemplateClass5::selectedSlaveItems() const
{
    return mSelectedSlaveItems;
}

/*!
 * \brief 从表当前选中条目的所有数据
 */
QVariantList TopTemplateClass5::selectedSlaveDataMaps() const
{
    return mSlaveView->selectedRowDataMaps(true);
}

/*!
 * \brief 导入树形结构的数据
 * {
 *   "A": {
 *     "master_title": "",
 *     "data": [
 *       {
 *         "slave_id": "",
 *         "slave_name": ""
 *       }
 *     ]
 *   }
 * }
 */
void TopTemplateClass5::loadTreeData(const QVariantMap &iTreeData,
                                     const QString &iSlavesKey)
{
    mMasterData.clear();
    mMasterSlaveDataMap.clear();
    QMapIterator<QString, QVariant> i(iTreeData);
    while (i.hasNext()) {
        i.next();
        QVariantMap item = i.value().toMap();
        mMasterSlaveDataMap.insert(i.key(), item.value(iSlavesKey));
        item.remove(iSlavesKey);
        item.insert("name", i.key());
        mMasterData.append(item);
    }
    refreshMaster();
}

/*!
 * \brief 导入表格结构的数据
 * \todo 待实现
 */
void TopTemplateClass5::loadTableData(const QVariantList &iTableData,
                                      const QString &iSlavesKey)
{
    Q_UNUSED(iTableData)
    Q_UNUSED(iSlavesKey)
    // @todo
}

void TopTemplateClass5::loadMasterData(const QVariantList &iMasterData, int iRowCount)
{
    mMasterData = iMasterData;
    if(mMasterPageTool) {
        mMasterPageTool->setRowCount(iRowCount,true);
    }
}

void TopTemplateClass5::insertSlaveData(const QString &iKey, const QVariant &iValue)
{
    mMasterSlaveDataMap.insert(iKey, iValue);
}

void TopTemplateClass5::insertSlaveCount(const QString &iKey, int iCount)
{
    mMasterSlaveCount.insert(iKey, iCount);
}

TUiLoader *TopTemplateClass5::uiLoader() const
{
    return mUiLoader;
}

void TopTemplateClass5::selectSlaveRow(const QVariant &iKeyVar)
{
    mSlaveView->selectRow(iKeyVar);
}

/*!
 * \brief 返回详细信息模块是否发生数据改变
 */
bool TopTemplateClass5::isDetailModified() const
{
    return mIsDetailModified;
}

/*!
 * \brief 设置详细信息的数据改变状态
 */
void TopTemplateClass5::setIsDetailModified(bool iIsDetailModified)
{
    if (mIsDetailModified == iIsDetailModified) {
        return;
    }
    mIsDetailModified = iIsDetailModified;
    emit isDetailModifiedChanged(iIsDetailModified);
}

QString TopTemplateClass5::naviWhere()
{
    QStringList naviSqlLst;
    if (mNaviType == NaviType::CategoryTreeView) {
        if (!mCategoryWhere.isEmpty()) {
            naviSqlLst.append(mCategoryWhere.toSql());
        }
    } else if (mNaviType == NaviType::UiLoader) {
        QString whereStr = uiloaderWhere();
        if (!whereStr.isEmpty()) {
            naviSqlLst.append(whereStr);
        }
    } else if (mNaviType == NaviType::AdvancedQuery) {
        Q_ASSERT_X(false, "TopTemplateClass5", "This feature is not yet supported.");
    } else if (mNaviType == NaviType::NaviFilter) {
        Q_ASSERT_X(false, "TopTemplateClass5", "This feature is not yet supported.");
    } else if (mNaviType == NaviType::CategoryTreeViewAndAdvancedQuery) {
        if (!mCategoryWhere.isEmpty()) {
            naviSqlLst.append(mCategoryWhere.toSql());
        }
        TAdvancedQuery *advancedQuery = findAdvancedQuery();
        if (advancedQuery != nullptr) {
            if (!advancedQuery->getSql().isEmpty()) {
                naviSqlLst.append(advancedQuery->getSql());
            }
        }
    } else if (mNaviType == NaviType::UiLoaderAndCategoryTreeView) {
        QString whereStr = uiloaderWhere();
        if (!whereStr.isEmpty()) {
            naviSqlLst.append(whereStr);
        }
        if (!mCategoryWhere.isEmpty()) {
            naviSqlLst.append(mCategoryWhere.toSql());
        }
    } else {
        // do nothing
    }
    return naviSqlLst.join(" AND ");
}

/*!
 * \brief 当主表选中发生改变的槽函数
 */
void TopTemplateClass5::onMasterSelectionChanged()
{
    mSelectedMasterItems = mMasterView->selectedPrimaryKeys();
    refreshSlave();
    refreshActionState();
}

/*!
 * \brief 当主表分页发生改变的槽函数
 */
void TopTemplateClass5::onMasterPageChanged()
{
    refreshMaster(false);
}

/*!
 * \brief 当从表选中发生改变的槽函数
 */
void TopTemplateClass5::onSlaveSelectionChanged()
{
    mSelectedSlaveItems = mSlaveView->selectedPrimaryKeys();
    refreshDetail();
    refreshActionState();
}

/*!
 * \brief 当从表分页发生改变的槽函数
 */
void TopTemplateClass5::onSlavePageChanged()
{
    refreshSlave(false);
}

void TopTemplateClass5::onDetailChanged(bool iIsDetailModified)
{
    if (iIsDetailModified) {
        showMaskFrame(mLeftSplitter);
    } else {
        hideMaskFrame();
    }
    refreshActionState();
}

/*!
 * \brief 当导航栏过滤选项发生改变时的槽函数
 */
void TopTemplateClass5::onCategoryViewDataChanged(const QVariantList iDataLst)
{
    mCategoryWhere.clear();
    QString format = config("navi.format", "filter_by_enum").toString();
    if (!iDataLst.isEmpty()) {
        if (format.compare("filter_by_enum") == 0) {
            QVariantMap cateMap;
            for (QVariant item: iDataLst) {
                QVariantMap itemMap = item.toMap();
                QString data = itemMap.value("data").toString();
                QString name = itemMap.value("name").toString();
                if (data.isEmpty()) {
                    continue;
                }
                cateMap.insertMulti(data,name);
            }
            for (QString field: cateMap.uniqueKeys()) {
                QString dbField = field;
                TSqlWhereCompsiteV2 childWhere;
                childWhere.setLogic(TSqlWhereCompsiteV2::Logic_Or);
                for (QVariant ele: cateMap.values(field)) {
                    QString value = ele.toString();
                    if (mMasterTableConf.fieldFormatMap.contains(dbField)) {
                        QString fieldValue = mMasterTableConf.fieldFormatMap.value(dbField).toString();
                        if (fieldValue.compare("array") == 0) {
                            QString sqlStr = QString("%1 @> '{%2}' ").arg(dbField).arg(value);
                            childWhere.append(sqlStr);
                        }
                    } else {
                        childWhere.append(dbField, value);
                    }
                }
                mCategoryWhere.append(childWhere);
            }
        } else if (format.compare("filter_by_sql_where") == 0) {
            QVariantMap item = iDataLst.at(0).toMap();
            QString sqlWhere = item.value("data").toString();
            if (!sqlWhere.isEmpty()) {
                mCategoryWhere.append(sqlWhere);
            }
        }
    }
    refreshMaster();
}

/**
* \brief 当高级查询界面的添加按钮触发时的槽函数
*/
void TopTemplateClass5::onAddActionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    TTableChooserDialog *dialog = new TTableChooserDialog(this);
    dialog->setTitle(tr("Please select field:"));
    dialog->setButtons(QStringList()
                       << ttr("Ok") + ":Ok:Yes"
                       << ttr("Cancel") + ":Cancel:Cancel"
                       << QString("%1:UnselectAll:ResetRole").arg(tr("Clear selection")));
    QVariantList headeritems;
    headeritems << QVariant() << TDataParse::stringList2VariantMap(
                       QStringList() << "name" << "title" << "display" << tr("Field") << "displayRole" << "$title"<<"resizeMode" << "Stretch");
    dialog->setHeaderItem(headeritems);
    dialog->setDataKeyList(QStringList() << "title" << "name");
    dialog->setPrimaryKey("name");
    dialog->setSearchKeys(QStringList() << "title");
    dialog->setLeftTableSelectionMode(QAbstractItemView::ExtendedSelection);
    dialog->setRightTableSelectionMode(QAbstractItemView::ExtendedSelection);
    dialog->tableView()->verticalHeader()->setVisible(false);
    dialog->selectedTableView()->verticalHeader()->setVisible(false);
    dialog->loadAllData(getAdvancedQueryFieldList());
    dialog->loadSelectedData(advancedQuery->getQueryData().value("value").toList());

    QVariantList ret = dialog->run();
    if (ret.isEmpty()) {
        return;
    }

    QStringList conditionLst;
    for (QVariant var : ret) {
        QString fieldName = var.toMap().value("name").toString();
        conditionLst << fieldName;
    }
    advancedQuery->clearSearchCondition();
    advancedQuery->addSearchCondition(conditionLst);
}

/**
* \brief 当高级查询界面的清除按钮触发时的槽函数
*/
void TopTemplateClass5::onClearActionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }
    advancedQuery->clearSearchCondition();
}

/**
* \brief 当高级查询界面的另存为按钮触发时的槽函数
*/
void TopTemplateClass5::onSaveAsActionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    QString fileName = QFileDialog::getSaveFileName(this,
                                                    tr("Save query criteria as file"),
                                                    QString(),
                                                    QString("*.cond"));
    if (fileName.isEmpty()) {
        return;
    }

    QVariantMap newConditionMap;
    newConditionMap.insert("condition", advancedQuery->getQueryData());
    QString conditionStr = TDataParse::variant2JsonStr(newConditionMap);
    TFileIo::writeFileString(conditionStr , fileName);
}

/**
* \brief 当高级查询界面的打开按钮触发时的槽函数
*/
void TopTemplateClass5::onOpenActtionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    QString fileName = QFileDialog::getOpenFileName(this,
                                                    tr("Open query criteria from file"),
                                                    ".",
                                                    tr("Condition file(*.cond)"));
    if (fileName.isEmpty()) {
        return;
    }
    QString conditionStr = TFileIo::readFileString(fileName);
    QVariantMap dataMap = TDataParse::jsonStr2Variant(conditionStr).toMap();
    QVariantMap conditionMap = dataMap.value("condition").toMap();
    advancedQuery->setData(conditionMap);
}

/**
* \brief 当高级查询界面的保存按钮触发时的槽函数
*/
void TopTemplateClass5::onSaveAsCommonQueryActionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    TTableView *commonQueryTable = findCommonQueryTable();
    if (commonQueryTable == nullptr) {
        return;
    }

    QString uiStr;
    uiStr = "function func(self) {"
                   " return [{"
                           " name: 'basic_formlayout',"
                            "type: 'FormLayout',"
                            "property:{"
                                    "label_alignment: 'Top | Right',"
                                    "horizontal_spacing: 20, vertical_spacing: 10, margin: 20"
                            "},"
                            "child: [{"
                                    "name: 'name',"
                                    "type: 'LineEdit',"
                                    "title: '" + tr("Query Name") + "',"
                                    "pack: {"
                                            "label: '" + tr("Query Name") + "'"
                                    "},"
                                    "validate: 'NOTNULL'"
                                    "}]"
                            "}]}";

    TUiloaderDialog *dialog = new TUiloaderDialog(this);
    dialog->setTitle(tr("Save as a common query"));
    dialog->setSelf(this);
    dialog->resize(250,100);
    dialog->setButtons(QStringList() << QString("%1:Ok:AcceptRole").arg(ttr("Ok")) << QString("%1:Cancel::RejectRole").arg(ttr("Cancel")));
    dialog->setUiStr(uiStr);

    QVariantMap ret = dialog->run(true);
    if (ret.isEmpty()) {
        return;
    }

    // 判断name是否已经存在
    QString conditionName = ret.value("name").toString();
    QVariantList conditionList = mAdvancedQueryTableDataMap.value("advance").toList();
    for (QVariant var: conditionList) {
        QVariantMap itemMap = var.toMap();
        if (conditionName == itemMap.value("title")) {
            TMessageBox::error(this, tr("Save as common condition"), tr("This name already exists!"));
            return;
        }
    }

    // 取新的查询条件
    QVariantMap conditionMap = advancedQuery->getQueryData();
    QVariantMap newConditionMap;
    newConditionMap.insert("title", conditionName);
    newConditionMap.insert("condition", conditionMap);
    conditionList << QVariant(newConditionMap);
    mAdvancedQueryTableDataMap.insert("advance", conditionList);

    // 生成json格式数据
    QString newConditionStr = TDataParse::variant2JsonStr(mAdvancedQueryTableDataMap);
    TSqlInserterV2 inserter;
    inserter.setTable("sys_conf");
    QVariantMap saveMap;
    saveMap.insert("data", newConditionStr);
    saveMap.insert("path", QStringList() << this->moduleName() << "AdvancedQuery");
    saveMap.insert("version", 1); // 因为sys_conf的version字段不能为空，因此这里设为1
    inserter.setData(saveMap);
    inserter.setField(saveMap.uniqueKeys());
    inserter.setAutoIncrementField("id");
    inserter.uniqueFieldRef() << "path";

    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_REPLACE_ROW, QVariant::fromValue(inserter));
    TDataResponse dataRes(data.toMap());
    if (dataRes.hasError()) {
        alertError(ttr("Load data failed!"), dataRes.errText());
        return;
    } else {
        commonQueryTable->loadData(conditionList);
    }
}

/**
* \brief 当常用查询界面的表格双击条目时的槽函数
*/
void TopTemplateClass5::onCommonQueryTableDoubleClicked(const QModelIndex &index)
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    TTableView *commonQueryTable = findCommonQueryTable();
    if (commonQueryTable == nullptr) {
        return;
    }

    TTabWidget *advancedQueryTabWidget = findAdvancedQueryTabWidget();
    if (advancedQueryTabWidget == nullptr) {
        return;
    }

    advancedQuery->clearSearchCondition();
    TTableModel *tableModel = qobject_cast<TTableModel *>(commonQueryTable->model());
    QVariantMap rowDataMap = tableModel->rowDataMap(index);
    QVariantMap conditionMap = rowDataMap.value("condition").toMap();
    advancedQuery->setData(conditionMap);
    advancedQueryTabWidget->setCurrentIndex(0);
}

/**
* \brief 当常用查询界面的表格删除条目时的槽函数
*/
void TopTemplateClass5::onDeleleCommonQueryActtionTriggered()
{
    TTableView *commonQueryTable = findCommonQueryTable();
    if (commonQueryTable == nullptr) {
        return;
    }

    QVariantMap selectedConditionMap = commonQueryTable->currentRowDataMap().toMap();
    if (selectedConditionMap.isEmpty()) {
        return;
    }

    QString selectedTitle = selectedConditionMap.value("title").toString();
    QVariantList conditionList = mAdvancedQueryTableDataMap.value("advance").toList();;
    QVariantList conditionDataList;
    for (QVariant var : conditionList){
        QString title = var.toMap().value("title").toString();
        // 如果不是选中的项，表示需要保留
        if (title != selectedTitle) {
            conditionDataList << QVariant(var.toMap());
        }
    }
    commonQueryTable->loadData(conditionDataList);
    mAdvancedQueryTableDataMap.insert("advance", conditionDataList);

    //生成json格式数据
    QString newConditionStr = TDataParse::variant2JsonStr(mAdvancedQueryTableDataMap);
    TSqlInserterV2 inserter;
    inserter.setTable("sys_conf");
    QVariantMap saveMap;
    saveMap.insert("data", newConditionStr);
    saveMap.insert("path", QStringList() << this->moduleName() << "AdvancedQuery");
    inserter.setData(saveMap);
    inserter.setField(saveMap.uniqueKeys());
    inserter.setAutoIncrementField("id");
    inserter.uniqueFieldRef() << "path";
    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_REPLACE_ROW, QVariant::fromValue(inserter));
    TDataResponse dataRes(data.toMap());
    if (dataRes.hasError()) {
        alertError(ttr("Load data failed!"), dataRes.errText());
        return;
    }
}

void TopTemplateClass5::onNaviReset()
{
    TUiLoader *filterUi = nullptr;
    if (mNaviType == NaviType::UiLoaderAndCategoryTreeView) {
        filterUi = mNaviViewUi;
    } else {
        filterUi = qobject_cast<TUiLoader*>(mNaviView);
    }
    if (filterUi == nullptr) {
        return;
    }
    filterUi->clearValues();
    this->refreshMaster();
}


/*!
 * \brief 初始化界面
 */
void TopTemplateClass5::initUi()
{
    mBodySplitter = new TSplitter(this);
    mBodySplitter->setObjectName("mBodySplitter");
    setCentralWidget(mBodySplitter);

    mLeftSplitter = new TSplitter(this);
    mLeftSplitter->setObjectName("mLeftSplitter");
    mBodySplitter->addWidget(mLeftSplitter);

    initNaviView();
    // 主表
    QFrame *masterWgt = new QFrame(this);
    mLeftSplitter->addWidget(masterWgt);

    QVBoxLayout *masterLayout = new QVBoxLayout(masterWgt);
    masterLayout->setMargin(0);
    masterLayout->setSpacing(0);

    mMasterView = new TTableView(this);
    mMasterView->setObjectName("mMasterView");
    connect(mMasterView->horizontalHeader(), SIGNAL(sortIndicatorChanged(int,Qt::SortOrder)),
            this, SLOT(refreshMaster()));
    connect(mMasterView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SLOT(onMasterSelectionChanged()));
    mMasterView->setDataKeyList(mMasterTableConf.dataKeys);
    mMasterView->setPrimaryKey(mMasterTableConf.primaryKey);
    QVariantList masterHeaderItems = mMasterTableConf.horizontalHeaders;
    masterHeaderItems.prepend(QVariant());
    mMasterView->setHeaderItem(masterHeaderItems);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("MASTER_TOOLBAR"))) {
        masterLayout->addWidget(toolbar, 0);
        mMasterSearchEntry = qobject_cast<TSearchEntry *>(uim()->getWidget("MASTER_TOOLBAR/SEARCH_ENTRY"));
        if (mMasterSearchEntry != nullptr) {
            connect(mMasterSearchEntry, SIGNAL(search(QString,QVariant)), this, SLOT(refreshMaster()));
            mMasterSearchEntry->setOptionList(TDataParse::headerItem2searchList(mMasterView->headerItem()));
            mMasterSearchEntry->setPlaceholderText(ttr("Search %1"));
        }
    }
    if (QMenu *tablePopup = qobject_cast<QMenu *>(uim()->getWidget("MASTER_POPUP"))) {
        mMasterView->setContextMenu(tablePopup);
    }

    masterLayout->addWidget(mMasterView, 1);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("MASTER_BOTTOM_TOOLBAR"))) {
        masterLayout->addWidget(toolbar, 0);
        mMasterPageTool = qobject_cast<TPageTool *>(uim()->getWidget("MASTER_BOTTOM_TOOLBAR/PAGE_TOOL"));
        if (mMasterPageTool != nullptr) {
            mMasterPageTool->setPageSizeVisible(true);
            connect(mMasterPageTool, SIGNAL(pageChanged(int,int)), this, SLOT(onMasterPageChanged()));
        }
    }

    // 从表
    QWidget *slaveWgt = new QWidget(this);
    mLeftSplitter->addWidget(slaveWgt);

    QVBoxLayout *slaveLayout = new QVBoxLayout(slaveWgt);
    slaveLayout->setMargin(0);
    slaveLayout->setSpacing(0);

    mSlaveView = new TTableView(this);
    mSlaveView->setObjectName("mSlaveView");
    connect(mSlaveView->horizontalHeader(), SIGNAL(sortIndicatorChanged(int,Qt::SortOrder)),
            this, SLOT(refreshSlave()));
    connect(mSlaveView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SLOT(onSlaveSelectionChanged()));
    mSlaveView->setDataKeyList(mSlaveTableConf.dataKeys);
    mSlaveView->setPrimaryKey(mSlaveTableConf.primaryKey);
    QVariantList slaveHeaderItems = mSlaveTableConf.horizontalHeaders;
    slaveHeaderItems.prepend(QVariant());
    mSlaveView->setHeaderItem(slaveHeaderItems);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("SLAVE_TOOLBAR"))) {
        slaveLayout->addWidget(toolbar, 0);
        mSlaveSearchEntry = qobject_cast<TSearchEntry *>(uim()->getWidget("SLAVE_TOOLBAR/SEARCH_ENTRY"));
        if (mSlaveSearchEntry != nullptr) {
            connect(mSlaveSearchEntry, SIGNAL(search(QString,QVariant)), this, SLOT(refreshSlave()));
            mSlaveSearchEntry->setOptionList(TDataParse::headerItem2searchList(mSlaveView->headerItem()));
            mSlaveSearchEntry->setPlaceholderText(ttr("Search %1"));
        }
    }
    if (QMenu *tablePopup = qobject_cast<QMenu *>(uim()->getWidget("SLAVE_POPUP"))) {
        mSlaveView->setContextMenu(tablePopup);
    }

    slaveLayout->addWidget(mSlaveView, 1);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("SLAVE_BOTTOM_TOOLBAR"))) {
        slaveLayout->addWidget(toolbar, 0);
        mSlavePageTool = qobject_cast<TPageTool *>(uim()->getWidget("SLAVE_BOTTOM_TOOLBAR/PAGE_TOOL"));
        if (mSlavePageTool != nullptr) {
            mSlavePageTool->setPageSizeVisible(true);
            connect(mSlavePageTool, SIGNAL(pageChanged(int,int)), this, SLOT(onSlavePageChanged()));
        }
    }

    // 详情界面
    const QString uiStr = ui("detail").toString();
    if(!uiStr.isEmpty()) {
        QWidget *detailWgt = new QWidget(this);
        mBodySplitter->addWidget(detailWgt);
        QVBoxLayout *detailVLayout = new QVBoxLayout(detailWgt);
        detailVLayout->setMargin(0);
        detailVLayout->setSpacing(0);

        if(QToolBar *toolBar = qobject_cast<QToolBar*>(uim()->getWidget("DETAIL_TOOLBAR"))) {
            detailVLayout->addWidget(toolBar);
        }

        mUiLoader = new TUiLoader(detailWgt);
        const QStringList typePlugins = config("uiloader_plugins").toStringList();
        if (typePlugins.size() > 0) {
            TopClassHelper::setUiLoaderPluginCallback(mUiLoader,this,typePlugins);
        }
        mUiLoader->setProperty("SS_BG","PANEL");
        mUiLoader->setSelf(this);
        mUiLoader->setScriptEngine(APP->scriptEngine());
        mUiLoader->setUiStr(uiStr);
        connect(mUiLoader, &TUiLoader::dataChanged, [this](){
            setIsDetailModified(true);
        });
        detailVLayout->addWidget(mUiLoader);
    }

    resizeSplitterByRatio();
}

/*!
 * \brief 初始化导航栏
 */
void TopTemplateClass5::initNaviView()
{
    mNaviType = str2NaviType(config("navi.__type__", "None").toString());
    if (mNaviType == NaviType::None) {
        return;
    }

    TAccordion *accordion = new TAccordion(this);
    // 此ObjectName在其他逻辑中是有意义的，请勿删除
    accordion->setObjectName("__navi__");
    accordion->setProperty("SS_BG","NAVI");
    accordion->setMinimumWidth(TTHEME_DP(config("navi.min_width", 150).toInt()));
    accordion->setMaximumWidth(TTHEME_DP(config("navi.max_width", 300).toInt()));
    accordion->setAccordionLocation(TAccordion::AccordionLocation::Left);
    QVBoxLayout *naviLayout = new QVBoxLayout(accordion);
    naviLayout->setMargin(0);
    naviLayout->setSpacing(0);
    QLabel *naviLabel = new QLabel(tr("Navigation"));
    naviLabel->setProperty("SS_TYPE", "SUBHEADER");
    naviLayout->addWidget(naviLabel,0);
    naviLayout->addSpacing(TTHEME_DP(4));
    accordion->expandButton()->setFixedHeight(TTHEME_DP(40));
    accordion->expandButton()->setToolTip(tr("Show Navigation"));
    accordion->collapseButton()->setToolTip(tr("Hide Navigation"));
    accordion->setIsExpanded(config("navi.is_expanded", false).toBool());
    mLeftSplitter->insertWidget(0,accordion);

    if (mNaviType == NaviType::UiLoader) {
        mNaviView = new TUiLoader(this);
        TUiLoader *filterUi = qobject_cast<TUiLoader*>(mNaviView);
        filterUi->setSelf(this);
        filterUi->setScriptEngine(APP->scriptEngine());
        filterUi->setUiStr(ui("navi").toString());

        QFrame *naviWgt = new QFrame(this);
        QVBoxLayout *leftLayout = new QVBoxLayout(naviWgt);
        leftLayout->addWidget(mNaviView);
        QHBoxLayout *leftBtnsLayout = new QHBoxLayout();
        leftBtnsLayout->setMargin(0);
        leftBtnsLayout->setSpacing(10);
        leftLayout->addLayout(leftBtnsLayout);
        QPushButton *resetBtn = new QPushButton(this);
        connect(resetBtn, SIGNAL(clicked(bool)), this, SLOT(onNaviReset()));
        leftBtnsLayout->addWidget(resetBtn);
        resetBtn->setText(tr("Reset"));
        resetBtn->setIcon(TRES->icon("rotate-right"));
        QPushButton *filterBtn = new QPushButton(this);
        connect(filterBtn, SIGNAL(clicked(bool)), this, SLOT(refreshMaster(bool)));
        leftBtnsLayout->addWidget(filterBtn);
        filterBtn->setText(tr("Filter"));
        filterBtn->setIcon(TRES->icon("filter"));
        leftLayout->addStretch(1);
        naviLayout->addWidget(naviWgt);
    } else if (mNaviType == NaviType::AdvancedQuery) {
        Q_ASSERT_X(false, "TopTemplateClass5", "This feature is not yet supported.");
    } else if (mNaviType == NaviType::NaviFilter) {
        Q_ASSERT_X(false, "TopTemplateClass5", "This feature is not yet supported.");
    } else if (mNaviType == NaviType::CategoryTreeViewAndAdvancedQuery) {
        // tab1-CategoryTreeView
        TCategoryTreeView *categoryTreeView = new TCategoryTreeView(this);
        categoryTreeView->setStyleSheet("TCategoryTreeView{border:NONE;background-color:transparent}");
        QVariantList rootChildList = parseCategoryConfig();
        if (rootChildList.isEmpty()) {
            mNaviView->setVisible(false);
            return;
        }
        categoryTreeView->setItemList(rootChildList);
        categoryTreeView->setCheckable(config("navi.is_checkable").toBool());
        connect(categoryTreeView, SIGNAL(selectionChanged(QVariantList)),
                this, SLOT(onCategoryViewDataChanged(QVariantList)));

        // tab2-tab1-ToolBar
        QToolBar *toolbar = new QToolBar(this);
        toolbar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        toolbar->setProperty("SS_SIZE", "SMALL");
        toolbar->setMovable(false);

        QAction *queryAction = toolbar->addAction(APP->getResIcon("search"), tr("Query"));
        connect(queryAction, SIGNAL(triggered()), this, SLOT(refreshMaster()));
        toolbar->addSeparator();

        QAction* addAction = toolbar->addAction(APP->getResIcon("addto-circle-o"), tr("Add"));
        addAction->setToolTip(tr("Add query criteria"));
        connect(addAction, SIGNAL(triggered()), this, SLOT(onAddActionTriggered()));

        QAction* clearAction = toolbar->addAction(APP->getResIcon("eraser"), tr("Clear"));
        clearAction->setToolTip(tr("Clear all query criteria"));
        connect(clearAction, SIGNAL(triggered()), this, SLOT(onClearActionTriggered()));

        QAction* saveAsAction = toolbar->addAction(APP->getResIcon("save"), tr("Save As"));
        saveAsAction->setToolTip(tr("Save query criteria as file"));
        connect(saveAsAction, SIGNAL(triggered()), this, SLOT(onSaveAsActionTriggered()));

        QAction* openActtion = toolbar->addAction(APP->getResIcon("folder-open-o"), tr("Open"));
        openActtion->setToolTip(tr("Open query criteria from file"));
        connect(openActtion, SIGNAL(triggered()), this, SLOT(onOpenActtionTriggered()));

        QAction* saveAsCommonQueryAction = toolbar->addAction(APP->getResIcon("save-as"), tr("Save"));
        saveAsCommonQueryAction->setToolTip(tr("Save as a common query"));
        connect(saveAsCommonQueryAction, SIGNAL(triggered()), this, SLOT(onSaveAsCommonQueryActionTriggered()));

        // tab2-tab1-AdvancedQuery
        TAdvancedQuery *advancedQuery = new TAdvancedQuery(this);
        advancedQuery->setObjectName("AdvancedQuery");
        initAdvanceCfgMap();
        advancedQuery->setItemList(mAdvancedQueryCfgMap.value("advanced_items").toList());
        advancedQuery->setData(getDefaultQueryDataFromCfgMap());

        TScrollArea *scrollArea = new TScrollArea(this);
        scrollArea->setWidgetResizable(true);
        scrollArea->setWidget(advancedQuery);

        // tab2-tab1
        QMainWindow *mainWindow = new QMainWindow(this);
        mainWindow->addToolBar(toolbar);
        mainWindow->setCentralWidget(scrollArea);

        // tab2-tab2-TableView
        TTableView *commonQueryTable = new TTableView(this);
        commonQueryTable->setObjectName("CommonQueryTable");
        commonQueryTable->setShowGrid(false);
        commonQueryTable->setSelectionMode(QAbstractItemView::ExtendedSelection);
        commonQueryTable->verticalHeader()->setVisible(false);
        QVariantList headerItem;
        headerItem << QVariant() << TDataParse::variantList2Map(
                          QVariantList() <<"name" << "title" << "display" << tr("Query Name") << "resizeMode" << "Stretch" << "displayRole" << "$title");
        commonQueryTable->setHeaderItem(headerItem);
        commonQueryTable->setDataKeyList(QStringList() << "title" << "condition");
        commonQueryTable->setPrimaryKey("condition");
        connect(commonQueryTable, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(onCommonQueryTableDoubleClicked(const QModelIndex &)));

        QAction *deleleCommonQueryActtion = new QAction(APP->getResIcon("remove"), tr("Delete query"), this);
        connect(deleleCommonQueryActtion, SIGNAL(triggered()), this, SLOT(onDeleleCommonQueryActtionTriggered()));
        QMenu *contextMenu = new QMenu(this);
        contextMenu->addAction(deleleCommonQueryActtion);
        commonQueryTable->setContextMenu(contextMenu);

        // 刷新常用查询表格数据
        initCommonQueryMap();
        commonQueryTable->loadData(mAdvancedQueryTableDataMap.value("advance").toList());

        // 高级查询Tab
        TTabWidget *advancedQueryTabWidget = new TTabWidget(this);
        advancedQueryTabWidget->setObjectName("AdvancedQueryTabWidget");
        advancedQueryTabWidget->addTab(mainWindow, tr("Advanced Query"));
        advancedQueryTabWidget->addTab(commonQueryTable, tr("Common Query"));

        // 导航Tab
        mNaviView = new TTabWidget(this);
        TTabWidget *tabWidget = qobject_cast<TTabWidget *>(mNaviView);
        tabWidget->addTab(categoryTreeView, tr("Category"));
        tabWidget->addTab(advancedQueryTabWidget, tr("Advanced Query"));
        naviLayout->addWidget(mNaviView);
    } else if (mNaviType == NaviType::UiLoaderAndCategoryTreeView) {
        mNaviView = new QFrame(this);
        QVBoxLayout *leftLayout = new QVBoxLayout(mNaviView);
        // uiLoader
        mNaviViewUi = new TUiLoader(this);
        mNaviViewUi->setSelf(this);
        mNaviViewUi->setScriptEngine(APP->scriptEngine());
        mNaviViewUi->setUiStr(ui("navi").toString());
        leftLayout->addWidget(mNaviViewUi);
        QHBoxLayout *leftBtnsLayout = new QHBoxLayout();
        leftBtnsLayout->setMargin(0);
        leftBtnsLayout->setSpacing(10);
        leftLayout->addLayout(leftBtnsLayout);
        QPushButton *resetBtn = new QPushButton(this);
        connect(resetBtn, SIGNAL(clicked(bool)), this, SLOT(onNaviReset()));
        leftBtnsLayout->addWidget(resetBtn);
        resetBtn->setText(tr("Reset"));
        resetBtn->setIcon(TRES->icon("rotate-right"));
        QPushButton *filterBtn = new QPushButton(this);
        connect(filterBtn, SIGNAL(clicked(bool)), this, SLOT(refreshMaster(bool)));
        leftBtnsLayout->addWidget(filterBtn);
        filterBtn->setText(tr("Filter"));
        filterBtn->setIcon(TRES->icon("filter"));
        // categoryTreeView
        mNaviViewCat = new TCategoryTreeView(this);
        mNaviViewCat->setStyleSheet("TCategoryTreeView{border:NONE;background-color:transparent}");
        QVariantList rootChildList = parseCategoryConfig();
        if (rootChildList.isEmpty()) {
            mNaviViewCat->setVisible(false);
        }
        mNaviViewCat->setItemList(rootChildList);
        mNaviViewCat->setCheckable(config("navi.is_checkable").toBool());
        connect(mNaviViewCat, SIGNAL(selectionChanged(QVariantList)),
                this, SLOT(onCategoryViewDataChanged(QVariantList)));
        mNaviViewCat->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        leftLayout->addSpacing(6);
        leftLayout->addWidget(mNaviViewCat, 1);
        naviLayout->addWidget(mNaviView);
    } else {
        mNaviView = new TCategoryTreeView(this);
        mNaviView->setStyleSheet("TCategoryTreeView{border:NONE;background-color:transparent}");
        QVariantList rootChildList = parseCategoryConfig();
        if (rootChildList.isEmpty()) {
            mNaviView->setVisible(false);
            return;
        }
        TCategoryTreeView *categoryTreeView = qobject_cast<TCategoryTreeView *>(mNaviView);
        categoryTreeView->setItemList(rootChildList);
        categoryTreeView->setCheckable(config("navi.is_checkable").toBool());
        connect(categoryTreeView, SIGNAL(selectionChanged(QVariantList)),
                this, SLOT(onCategoryViewDataChanged(QVariantList)));
        naviLayout->addWidget(mNaviView);
    }
}

/**
* \brief 初始化高级查询配置数据
* mAdvancedQueryCfgMap结构如下：
* \code
    advance: {
      "advanced_items" : [
        {"name" : "attr_data.machine_code", "title" : "Machine Code" , "widgetType" : "LineEdit" , "wgt_prop" : "" , "optionList" : []},
        {"name" : "attr_data.machine_name", "title" : "Machine Name" , "widgetType" : "LineEdit" , "wgt_prop" : "" , "optionList" : []},
        {"name" : "title", "title" : "Title" , "widgetType" : "LineEdit" , "wgt_prop" : "" , "optionList" : []},
        {"name" : "class", "title" : "Class" , "widgetType" : "ComboBox" , "wgt_prop" : "" , "optionList" : "enum(tpm-machine-maintenance-plan-class)"}
      ],
      "file_suffix": ".conf"
      "condition" :{
        "condition": "1 and 2",
        "value": [
            {
                "ignoreEmpty": false,
                "name": "attr_data.machine_code",
                "operator": "LIKE",
                "title": "Machine Code",
                "valid": true,
                "value1": "00"
            },
            {
                "ignoreEmpty": false,
                "name": "attr_data.machine_name",
                "operator": "LIKE",
                "title": "Machine Name",
                "valid": true,
                "value1": "4"
            }
        ]
      }
    }
* \endcode
*/
void TopTemplateClass5::initAdvanceCfgMap()
{
    mAdvancedQueryCfgMap = config("advance").toMap();
    QVariantList queryItems = mAdvancedQueryCfgMap.value("advanced_items").toList();
    QVariantList newQueryItems;
    for (QVariant var : queryItems) {
        QVariantMap m = var.toMap();
        // 翻译title
        m.insert("title", ttr(m.value("title").toString()));
        // 转换枚举
        QVariantList optionList;
        QRegExp regExp("enum(.*)");
        if ((m.value("optionList").type() == QVariant::String) && (regExp.indexIn(m.value("optionList").toString()) != -1)) {
            if (regExp.capturedTexts().count() > 1) {
                optionList = TOPENM->enumList(regExp.capturedTexts().value(1).remove("(").remove(")"))->toComboList();
                m.insert("optionList", optionList);
            }
        }
        newQueryItems << m;
    }
    mAdvancedQueryCfgMap.insert("advanced_items", newQueryItems);
}

/**
* \brief 获取高级查询控件默认的查询条件
*/
QVariantMap TopTemplateClass5::getDefaultQueryDataFromCfgMap()
{
    QVariantMap conditionMap = config("advance").toMap().value("condition").toMap();
    QVariantList valueList = conditionMap.value("value").toList();
    for (QVariant &var: valueList) {
        QVariantMap m = var.toMap();
        m.insert("title", ttr(m.value("title").toString()));
        var = m;
    }
    conditionMap.insert("value", valueList);
    return conditionMap;
}

/**
* \brief 从高级查询配置中获取未使用条目的name和text，添加条目时使用
*/
QVariantList TopTemplateClass5::getAdvancedQueryFieldList()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return QVariantList();
    }

    QVariantList selectedFieldDataLst = advancedQuery->getQueryData().value("value").toList();
    QStringList selectedFieldLst;
    for (QVariant val : selectedFieldDataLst){
        selectedFieldLst.append(val.toMap().value("name").toString());
    }

    QVariantList queryItems = mAdvancedQueryCfgMap.value("advanced_items").toList();
    QVariantList fieldList;
    for (QVariant var : queryItems) {
        QVariantMap m = var.toMap();
        QVariantMap itemMap;
        if(!selectedFieldLst.contains(m.value("name").toString())) {
            itemMap.insert("name", m.value("name").toString());
            itemMap.insert("title", ttr(m.value("title").toString()));
            fieldList << QVariant(itemMap);
        }
    }
    return fieldList;
}

/**
* \brief 从sys_conf表查找出已保存的常用查询数据
* 数据结构如下：
* \code
{
  "advance": [
    {
      "condition": {
        "condition": "1 and 2",
        "value": [
          {
            "ignoreEmpty": false,
            "name": "attr_data.machine_code",
            "operator": "LIKE",
            "title": "设备编号",
            "valid": true,
            "value1": "00"
          },
          {
            "ignoreEmpty": false,
            "name": "attr_data.machine_name",
            "operator": "LIKE",
            "title": "设备名称",
            "valid": true,
            "value1": "4"
          }
        ]
      },
      "title": "1"
    }
  ]
}
* \endcode
*/
void TopTemplateClass5::initCommonQueryMap()
{
    TSqlSelectorV2 selector;
    selector.setTable("sys_conf");
    selector.setField(QStringList() << "data");
    selector.setWhere("path", QStringList() << this->moduleName() << "AdvancedQuery", "@>");
    TError err;
    QVariant data = runSqlQueryOnThreadSync(TOPSQLTHREAD_SELECT_MAP,
                                            QVariant::fromValue(selector),
                                            &err);
    if (err.isValid()) {
        alertError(ttr("Load data failed!"), err.text());
        return;
    }
    mAdvancedQueryTableDataMap = TDataParse::jsonStr2Variant(data.toMap().value("data").toString()).toMap();
}

/**
* \brief 根据objname从导航栏中找到高级查询控件
*/
TAdvancedQuery *TopTemplateClass5::findAdvancedQuery()
{
    return mNaviView->findChild<TAdvancedQuery *>("AdvancedQuery");
}

/**
* \brief 根据objname从导航栏中找到常用查询tableView控件
*/
TTableView *TopTemplateClass5::findCommonQueryTable()
{
    return mNaviView->findChild<TTableView *>("CommonQueryTable");
}

/**
* \brief 根据objname从导航栏中找到高级查询TabWidget控件
*/
TTabWidget *TopTemplateClass5::findAdvancedQueryTabWidget()
{
    return mNaviView->findChild<TTabWidget *>("AdvancedQueryTabWidget");
}

/*!
 * \brief 解析导航栏配置
 */
QVariantList TopTemplateClass5::parseCategoryConfig()
{
    QVariantList categoryLst;

    QVariantList categoryConfig = config("navi.categories").toList();
    QString format = config("navi.format", "filter_by_enum").toString();
    if (format.compare("filter_by_enum") == 0) {
        for(const QVariant &item: categoryConfig) {
            QVariantMap itemMap = item.toMap();
            QString itemName         = itemMap.value("name").toString();
            QVariantList childrenLst = itemMap.value("children").toList();
            QString enumCfg          = itemMap.value("enum_children").toString();
            QStringList enumCheckedLst = itemMap.value("enum_checked").toStringList();
            QStringList enumInvisibleLst = itemMap.value("enum_invisible").toStringList();
            if (itemMap.contains("visible")) {
                itemMap.insert("VISIBLE", itemMap.take("visible"));
            }
            if (itemMap.contains("expand")) {
                itemMap.insert("EXPAND", itemMap.take("expand"));
            }
            TSqlWhereCompsiteV2 childWhere;
            childWhere.setLogic(TSqlWhereCompsiteV2::Logic_Or);
            QVariantList type = TOPENM->enumList(enumCfg)->toComboList();
            for (const QVariant &item: type) {
                QVariantMap enumMap = item.toMap();
                QString enumName = enumMap.value("name").toString();
                QVariantMap childMap;
                childMap.insert("name",enumName);
                childMap.insert("text",ttr(enumMap.value("text").toString()));
                childMap.insert("icon",enumMap.value("icon"));
                childMap.insert("data", itemName);
                if(enumCheckedLst.contains(enumName)) {
                    childMap.insert("checked",1);
                } else {
                    childMap.insert("checked",0);
                }
                if (enumInvisibleLst.contains(enumName)) {
                    childMap.insert("VISIBLE", 0);
                } else {
                    childMap.insert("VISIBLE",1);
                }
                childrenLst.append(childMap);
            }
            // 1 根据checked组装CategoryTreeView的默认查询条件 2 根据checked算出父级的勾选状态
            int checkedChildLen = 0;
            for (QVariant &item : childrenLst) {
                QVariantMap itemMap = item.toMap();
                QString data = itemMap.value("data").toString();
                QString name = itemMap.value("name").toString();
                if(itemMap.value("checked").toInt() == 1) {
                    childWhere.append(data, name);
                    checkedChildLen++;
                }
                item = itemMap;
            }
            if (checkedChildLen == 0) {
                itemMap.insert("checked", 0); // 不勾选
            } else if (checkedChildLen == childrenLst.length()) {
                itemMap.insert("checked", 1); // 勾选
            } else {
                itemMap.insert("checked", -1); // 部分勾选
            }
            mCategoryWhere.append(childWhere);

            itemMap.remove("children");
            itemMap.remove("enum_children");
            itemMap.remove("enum_checked");
            itemMap.remove("enum_invisible");
            itemMap.insert("text",ttr(itemMap.value("text").toString()));
            itemMap.insert("CHILDREN",childrenLst);

            categoryLst.append(itemMap);
        }
    } else if (format.compare("filter_by_sql_where") == 0) {
        return categoryConfig;
    }
    return categoryLst;
}

TopTemplateClass5::NaviType TopTemplateClass5::str2NaviType(const QString &iType)
{
    const QString type = iType.toUpper();
    if (type == "UILOADER") {
        return NaviType::UiLoader;
    } else if (type == "ADVANCEDQUERY") {
        return NaviType::AdvancedQuery;
    } else if (type == "NAVIFILTER") {
        return NaviType::NaviFilter;
    } else if (type == "CATEGORYTREEVIEWANDADVANCEDQUERY") {
        return NaviType::CategoryTreeViewAndAdvancedQuery;
    } else if (type == "UILOADERANDCATEGORYTREEVIEW") {
        return NaviType::UiLoaderAndCategoryTreeView;
    } else if (type == "CATEGORYTREEVIEW") {
        return NaviType::CategoryTreeView;
    } else {
        return NaviType::None;
    }
}

QString TopTemplateClass5::uiloaderWhere() const
{
    TUiLoader *filterUi = nullptr;
    if (mNaviType == NaviType::UiLoaderAndCategoryTreeView) {
        filterUi = mNaviViewUi;
    } else {
        filterUi = qobject_cast<TUiLoader*>(mNaviView);
    }
    if (filterUi == nullptr) {
        return QString();
    }

    auto filterDataMap = filterUi->getAllValues().toVariant().toMap();

    QString searchSql;
    QMapIterator<QString, QVariant> iter(filterDataMap);
    while (iter.hasNext()) {
        iter.next();
        QString key = iter.key();
        QString value = iter.value().toString();
        if (value.isEmpty()) {
            continue;
        }
        auto obj = filterUi->getObject(key);
        if (obj != nullptr) {
            QVariantMap userData = obj->property("tui_user_data").toMap();
            if (userData.contains("field_name")) {
                key = userData.value("field_name").toString();
            }
            QString operatorStr = "=";
            if (userData.contains("operator")) {
                operatorStr = userData.value("operator").toString();
            }
            if (operatorStr.compare("like", Qt::CaseInsensitive) == 0 ||
                    operatorStr.compare("ilike", Qt::CaseInsensitive) == 0) {
                value = QString("'%%1%'").arg(value);
            } else if (operatorStr.compare("in", Qt::CaseInsensitive) == 0) {
                value = QString("(%1)").arg(value);
            } else {
                if (iter.value().type() == QVariant::String) {
                    value = QString("'%1'").arg(value);
                }
            }
            QString searchStr = QString("%1 %2 %3").arg(key).arg(operatorStr).arg(value);
            if (!searchSql.isEmpty()) {
                searchSql.append(" and ");
            }
            searchSql.append(searchStr);
        }
    }
    return searchSql;
}

void TopTemplateClass5::resizeSplitterByRatio()
{
    QString masterSlaveRatio = config("master_slave_ratio", "1:1:1").toString();
    QRegularExpression regExp("(?<master>\\d+):(?<slave>\\d+):(?<uiloader>\\d+)");
    QRegularExpressionMatch match = regExp.match(masterSlaveRatio);
    if (match.hasMatch()) {
        double m = match.captured("master").toDouble();
        double s = match.captured("slave").toDouble();
        double u = match.captured("uiloader").toDouble();
        if (mUiLoader == nullptr) {
            u = 0.0;
        }
        double leftRatio =  (m + s) / (m + s + u);
        int width = this->width();
        int leftWidth =  static_cast<int>(width * leftRatio);
        mBodySplitter->setSizes(QList<int>{ leftWidth, width-leftWidth });

        double leftRatio1 = m / (m + s);
        int leftWidth1 = leftWidth * leftRatio1;
        mLeftSplitter->setSizes(QList<int>{300, leftWidth1, leftWidth - leftWidth1 });
    }
}
