﻿#include "toptemplateclass8.h"
#include <QtWidgets>
#include <QFileDialog>
#include <QDesktopServices>
#include <QTextDocument>
#include <QTextBlock>
#include <QPdfWriter>
#include <QPainter>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tdataresponse.h>
#include <tbaseutil/tenumlist.h>
#include <tbaseutil/tfileio.h>
#include <tbaseutil/ttheme.h>
#include <tbaseutil/tresource.h>
#include <tdatabaseutil/tsqlqueryv2.h>
#include <tdatabaseutil/tsqlconnectionpoolv2.h>
#include <topcore/topcore.h>
#include <topcore/topclasssqlthread.h>
#include <topcore/topenummanager.h>
#include <toputil/t.h>
#include <twidget/tsplitter.h>
#include <twidget/ttableview.h>
#include <twidget/tcategorytreeview.h>
#include <twidget/tsearchentry.h>
#include <twidget/tpagetool.h>
#include <twidget/tuiloader.h>
#include <twidget/taccordion.h>
#include <twidget/tadvancedquery.h>
#include <twidget/tmessagebox.h>
#include <twidget/tscrollarea.h>
#include <twidget/ttabwidget.h>
#include <twidget/ttablechooserdialog.h>
#include <twidget/tuiloaderdialog.h>
#include <twidget/tfiledialog.h>
#include <twidget/tcombobox.h>
#include <tchart/tchartwidget.h>
#include <tchart/tchartstore.h>
#include <tchart/tchartaxisnumeric.h>
#include <texcelxs/excelobject.h>
#include <texcelxs/book.h>
#include <texcelxs/sheet.h>
#include <texcelxs/format.h>
#include <texcelxs/font.h>
#include <texcelxs/range.h>
#include <functional>


/*!
 * \class TopTemplateClass8
 * \inmodule templateclass
 * \brief 带导航过滤和图表(TChart)的单表展示。
 *
 * navi有六种类型：
 * navi.__type__: None                              无导航，这是默认值
 * navi.__type__: CategoryTreeView                  导航树
 * navi.__type__: UiLoader                          定制的过滤界面，需配置navi.ui.js
 * navi.__type__: AdvancedQuery                     高级查询（暂不支持）
 * navi.__type__: NaviFilter                        使用TNaviFilter控件的导航（暂不支持）
 * navi.__type__: CategoryTreeViewAndAdvancedQuery  导航树加高级查询
 * navi.__type__: UiLoaderAndCategoryTreeView       定制的过滤界面加导航树
 */

TopTemplateClass8::TopTemplateClass8(const QString &iModuleName,
                                     const QVariantMap &iUrlPars,
                                     QWidget *iParent)
    : TopClassAbs(iParent)
{
    initModule(iModuleName, iUrlPars);
    QString licenseKey = config("sys_license_key").toString();
    if (!licenseKey.isEmpty()) {
        setLicenseKey(licenseKey);
    }
    QStringList langLst = config("sys_lang_list").toStringList();
    for (const auto &lang : langLst) {
        appendLanguage(lang);
    }
    if (isHookExists("afterModuleInit")) {
        callHooks("afterModuleInit");
    }
    TopClassHelper::parseTableConf0(this, "view", &mTableConf);
    initUi();
    connect(this, SIGNAL(viewConfChanged()),
            this, SLOT(onViewConfChanged()));
    if (isHookExists("afterViewInit")) {
        callHooks("afterViewInit");
    }
    restoreSizeState();
    restoreObjectState(mBodySplitter);
    restoreObjectState(mTableView);

    setChartName(config("chart_name").toString());
    QTimer::singleShot(0, this, SLOT(refresh()));
    if (mNaviType == NaviType::CategoryTreeView) {
        (qobject_cast<TCategoryTreeView *>(mNaviView))->refresh();
    } else if (mNaviType == NaviType::UiLoaderAndCategoryTreeView) {
        mNaviViewCat->refresh();
    }
    refreshActionState();

    auto refreshInterval = config("view.timing_refresh_interval", 0).toInt();
    if (refreshInterval > 0) {
        mRefreshTimer = new QTimer(this);
        connect(mRefreshTimer, &QTimer::timeout, [=]() {
            this->callAction("refresh");
        });
        mRefreshTimer->start(refreshInterval * 1000);
    }
}

TopTemplateClass8::~TopTemplateClass8()
{
    saveSizeState();
    saveObjectState(mBodySplitter);
    saveObjectState(mTableView);

    if (isHookExists("onDestroy")) {
        callHooks("onDestroy");
    }
}

/*!
 * \brief 刷新数据，\a iResetPage 表示是否要重置分页
 */
void TopTemplateClass8::refresh(bool iResetPage)
{
    TSqlSelectorV2 selector;
    if (!mTableConf.dbSql.isEmpty()) {
        selector.setTable(QString("(%1) _TEMP_TABLE_").arg(mTableConf.dbSql));
    } else if (!mTableConf.dbTableName.isEmpty()) {
        selector.setTable(mTableConf.dbTableName);
    } else {
        // db_sql为空和db_table_name为空时不作处理
        return;
    }
    selector.setField(mTableConf.queryFields);
    selector.setFieldFormat(mTableConf.fieldFormatMap);
    selector.setReturnRowCount(true);
    // 搜索分页
    TopClassHelper::handleSearchPageOnQuery(mSearchEntry, mPageTool, iResetPage, &selector);
    // 排序
    TopClassHelper::handleOrderOnQuery(mTableView, &mTableConf, &selector);
    // 只获取有效数据
    if (!mTableConf.dbDelFlagKey.isEmpty()) {
        selector.addWhere(QString("%1 = 0 OR %1 IS NULL").arg(mTableConf.dbDelFlagKey));
    }
    // 过滤项
    if(!mTableConf.dbFilter.isEmpty()) {
        selector.addWhere(mTableConf.dbFilter);
    }
    // 导航栏过滤项
    if (mNaviType == NaviType::None) {
        // do nothing
    } else if (mNaviType == NaviType::UiLoader) {
        QString whereStr = uiloaderWhere();
        if (!whereStr.isEmpty()) {
            selector.addWhere(whereStr);
        }
    } else if (mNaviType == NaviType::AdvancedQuery) {
        Q_ASSERT_X(false, "TopTemplateClass8", "This feature is not yet supported.");
    } else if (mNaviType == NaviType::NaviFilter) {
        Q_ASSERT_X(false, "TopTemplateClass8", "This feature is not yet supported.");
    } else if (mNaviType == NaviType::CategoryTreeViewAndAdvancedQuery) {
        if (!mCategoryWhere.isEmpty()) {
            selector.addWhere(mCategoryWhere.toSql());
        }
        TAdvancedQuery *advancedQuery = findAdvancedQuery();
        if (advancedQuery != nullptr) {
            if (!advancedQuery->getSql().isEmpty()) {
                selector.addWhere(advancedQuery->getSql());
            }
        }
    } else if (mNaviType == NaviType::UiLoaderAndCategoryTreeView) {
        QString whereStr = uiloaderWhere();
        if (!whereStr.isEmpty()) {
            selector.addWhere(whereStr);
        }
        if (!mCategoryWhere.isEmpty()) {
            selector.addWhere(mCategoryWhere.toSql());
        }
    } else {
        if (!mCategoryWhere.isEmpty()) {
            selector.addWhere(mCategoryWhere.toSql());
        }
    }

    t::loading(this);
    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_ARRAYMAP, QVariant::fromValue(selector));
    unloading();
    TDataResponse response(data.toMap());
    if (response.hasError()) {
        alertError(ttr("Load data failed!"));
    } else {
        if (mPageTool != nullptr) {
            mPageTool->setRowCount(response.dataCount(), true);
        }
        QVariantList tableData = response.data().toList();
        TopClassHelper::formatTableData(this, &mTableConf, tableData);
        if (isHookExists("handleTableData")) {
            tableData = callHooksQuick("handleTableData", QVariantList() << QVariant(tableData)).toVariant().toList();
        }
        mTableView->loadData(tableData);
        mTableView->adjustViewSpan(mTableConf.mergeKeys);
        onClear();
        refreshActionState();
        alertOk(ttr("Data loaded"));
    }
}

/*!
 * \brief 刷新图表
 */
void TopTemplateClass8::refreshChart()
{
    if (mChartId.isEmpty()) {
        return;
    }
    t::loading(this, "", -1, 0);
    QString funcJsonData = TDataParse::variant2JsonStr(mFilterData);
    // 生成图表
    TError error;
    QVariantMap graphLayoutMap = TDataParse::jsonStr2Variant(mGraphLayout, &error).toMap();
    if (error.isValid()) {
        unloading();
        alertError(error.text());
        return;
    }
    // 图表布局种类，目前只支持GrdiLayout
    QString graphLayoutType = graphLayoutMap.value("type").toString().trimmed().toLower();
    QVariantList graphList = graphLayoutMap.value("child").toList();
    if (graphLayoutType == "gridlayout") {
        if (!mChartListWgt->layout()) {
            TGridLayout *chartLayout = new TGridLayout(mChartListWgt);
            chartLayout->setContentsMargins(2, 2, 2, 8);
            mChartListWgt->setLayout(chartLayout);
        }
        TSqlSelectorV2 selector;
        for (QVariant item : graphList) {
            QVariantMap itemMap = item.toMap();
            QString graphName = itemMap.value("name").toString().trimmed();
            QString graphType = itemMap.value("type").toString().trimmed().toLower();
            QVariantMap packMap = itemMap.value("pack").toMap();

            selector.clear();
            selector.setTable("pub_chart_graph");
            selector.setField(QStringList()<<"id"<<"dynamic");
            selector.setWhere("name", graphName);
            QVariant dataRetp = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_MAP, QVariant::fromValue(selector));
            TDataResponse dataResp(dataRetp.toMap());
            if (dataResp.hasError()) {
                unloading();
                alertError(ttr("Load data failed!"), dataResp.errText());
                return;
            }
            QVariantMap graphMap = dataResp.data().toMap();
            if (graphType == "uiloader") {
                generateUiLoader(graphMap.value("id").toInt(), funcJsonData, packMap, "gridlayout");
            } else {
                if (graphMap.value("dynamic").toInt() == 1) {
                    dynamicGenerateChart(graphMap.value("id").toInt(), funcJsonData, packMap, "gridlayout");
                } else {
                    configGenerateChart(graphMap.value("id").toInt(), funcJsonData, packMap, "gridlayout");
                }
            }
        }
    }
    unloading();
}

/*!
 * \brief 当前图表ID
 */
QString TopTemplateClass8::chartId() const
{
    return mChartId;
}

/*!
 * \brief 设置图表ID
 */
void TopTemplateClass8::setChartId(const QString &iId)
{
    if (mChartId != iId) {
        if (mChartListWgt != nullptr) {
            mChartListWgt->close();
            mChartListWgt->deleteLater();
            mChartListWgt = nullptr;
            for (TChartWidget *wid: mChartMap.values()) {
                if (wid) {
                    wid->setParent(nullptr);
                    wid->close();
                    wid->deleteLater();
                    wid = nullptr;
                }
            }
            mChartMap.clear();
            mChartParameterRemarkMap.clear();
            for (TUiLoader *loader: mUiLoaderMap.values()) {
                if (loader) {
                    loader->setParent(nullptr);
                    loader->close();
                    loader->deleteLater();
                    loader = nullptr;
                }
            }
            mUiLoaderMap.clear();
            mUiLoaderParamRemarkMap.clear();
        }
        mChartId = iId;
        mChartVisibleData.clear();
        if (mChartId.isEmpty()) return;
        TSqlSelectorV2 selector;
        selector.setTable("pub_chart_navigation");
        selector.setField(QStringList()<<"name"<<"parameter_list"<<"graph_layout");
        selector.setWhere("id", mChartId);
        QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_MAP, QVariant::fromValue(selector));
        TDataResponse dataRes(data.toMap());
        if (dataRes.hasError()) {
            alertError(dataRes.errText(), "Error");
        }
        QVariantMap naviData = dataRes.data().toMap();
        mChartName = naviData.value("name").toString();
        mGraphLayout = naviData.value("graph_layout").toString();
        mChartListWgt = new QWidget(mScrollArea);
        mScrollArea->setWidget(mChartListWgt);
        mScrollArea->setWidgetResizable(true);
    }
}

/*!
 * \brief 当前图表名称
 */
QString TopTemplateClass8::chartName() const
{
    return mChartName;
}

void TopTemplateClass8::setChartName(const QString &iName)
{
    if (mChartName != iName) {
        if (mChartListWgt != nullptr) {
            mChartListWgt->close();
            mChartListWgt->deleteLater();
            mChartListWgt = nullptr;
            for (TChartWidget *wid: mChartMap.values()) {
                if (wid) {
                    wid->setParent(nullptr);
                    wid->close();
                    wid->deleteLater();
                    wid = nullptr;
                }
            }
            mChartMap.clear();
            mChartParameterRemarkMap.clear();
            for (TUiLoader *loader: mUiLoaderMap.values()) {
                if (loader) {
                    loader->setParent(nullptr);
                    loader->close();
                    loader->deleteLater();
                    loader = nullptr;
                }
            }
            mUiLoaderMap.clear();
            mUiLoaderParamRemarkMap.clear();
        }
        mChartName = iName;
        mChartVisibleData.clear();
        if (mChartName.isEmpty()) return;
        TSqlSelectorV2 selector;
        selector.setTable("pub_chart_navigation");
        selector.setField(QStringList()<<"id"<<"parameter_list"<<"graph_layout");
        selector.setWhere("name", mChartName);
        QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_MAP, QVariant::fromValue(selector));
        TDataResponse dataRes(data.toMap());
        if (dataRes.hasError()) {
            alertError(dataRes.errText(), "Error");
        }
        QVariantMap naviData = dataRes.data().toMap();
        mChartId = naviData.value("id").toString();
        mGraphLayout = naviData.value("graph_layout").toString();
        mChartListWgt = new QWidget(mScrollArea);
        mScrollArea->setWidget(mChartListWgt);
        mScrollArea->setWidgetResizable(true);
    }
}

/*!
 * \brief 当前选中条目的primaryKey
 */
QVariantList TopTemplateClass8::selectedItems() const
{
    return mTableView->selectedPrimaryKeys();
}

/*!
 * \brief 当前选中条目的所有数据
 */
QVariantList TopTemplateClass8::selectedDataMaps() const
{
    return mTableView->selectedRowDataMaps(true);
}

/*!
 * \brief 删除条目
 */
void TopTemplateClass8::deleteItems(const QVariantList &iItems)
{
    if (mTableConf.dbDelFlagKey.isEmpty()) {
        // 物理删除
        TSqlDeleterV2 deleter;
        deleter.setTable(mTableConf.dbTableName);
        deleter.setWhere(mTableConf.primaryKey, iItems);
        TError err;
        runSqlQueryOnThreadSync(TOPSQLTHREAD_DELETE_ROW,
                                QVariant::fromValue(deleter),
                                &err);
        if (err.isValid()) {
            alertError(ttr("Delete data failed!"), err.text());
        } else {
            refresh(true);
            alertOk(ttr("Data deleted"));
        }
    } else {
        // 逻辑删除
        TSqlUpdaterV2 updater;
        updater.setTable(mTableConf.dbTableName);
        updater.setWhere(mTableConf.primaryKey, iItems);
        updater.setData(QVariantMap {{mTableConf.dbDelFlagKey, 1}});
        TError err;
        runSqlQueryOnThreadSync(TOPSQLTHREAD_UPDATE_ROW,
                                QVariant::fromValue(updater),
                                &err);
        if (err.isValid()) {
            alertError(ttr("Delete data failed!"), err.text());
        } else {
            refresh(true);
            alertOk(ttr("Moved to recycle bin"));
        }
    }
}

/*!
 * \brief 表格控件对象
 */
TTableView *TopTemplateClass8::tableView() const
{
    return mTableView;
}

/*!
 * \brief 导航控件
 */
QWidget *TopTemplateClass8::naviView() const
{
    return mNaviView;
}

/*!
 * \brief 导航树形控件
 */
TCategoryTreeView *TopTemplateClass8::naviTreeView() const
{
    TCategoryTreeView *catView = nullptr;
    if (mNaviType == NaviType::UiLoaderAndCategoryTreeView) {
        catView = mNaviViewCat;
    } else {
        catView = qobject_cast<TCategoryTreeView*>(mNaviView);
    }
    return catView;
}

/*!
 * \brief 导航UiLoader控件
 */
TUiLoader *TopTemplateClass8::naviLoader() const
{
    TUiLoader *filterUi = nullptr;
    if (mNaviType == NaviType::UiLoaderAndCategoryTreeView) {
        filterUi = mNaviViewUi;
    } else {
        filterUi = qobject_cast<TUiLoader*>(mNaviView);
    }
    return filterUi;
}

/*!
 * \brief 设置图表筛选参数
 * 与图表导航不同，这里没有图表上方的筛选界面，参数通过此接口设置
 */
void TopTemplateClass8::setFilterData(const QVariantMap &iDataMap)
{
    mFilterData = iDataMap;
}

void TopTemplateClass8::onDoubleClicked()
{
    if (TChartWidget *chartWgt = qobject_cast<TChartWidget*>(sender())) {
        TChartWidget *newChartWgt = new TChartWidget();
        newChartWgt->setAttribute(Qt::WA_DeleteOnClose, true);
        newChartWgt->setConfig(chartWgt->config());
        newChartWgt->setStore(chartWgt->store());
        newChartWgt->setWindowModality(Qt::ApplicationModal);
        newChartWgt->showMaximized();
    }
}

/*!
 * \brief 清空图表
 */
void TopTemplateClass8::onClear()
{
    if (mChartListWgt != nullptr) {
        for (TChartWidget *wid: mChartMap.values()) {
            if (wid) {
                wid->setParent(nullptr);
                wid->close();
                wid->deleteLater();
                wid = nullptr;
            }
        }
        mChartMap.clear();
        mChartParameterRemarkMap.clear();
        for (TUiLoader *loader: mUiLoaderMap.values()) {
            if (loader) {
                loader->setParent(nullptr);
                loader->close();
                loader->deleteLater();
                loader = nullptr;
            }
        }
        mUiLoaderMap.clear();
        mUiLoaderParamRemarkMap.clear();
    }
    mChartVisibleData.clear();
}

/*!
 * \brief 导出PDF
 * \param iPageSize A0-A9 B0-B10
 */
void TopTemplateClass8::onExportPdfClicked(const QString &iPageSize)
{
    if (mChartMap.size() == 0 && mUiLoaderMap.size() == 0) {
        TMessageBox::error(this, tr("No chart to export"));
        return;
    }

    QString defaultPath = QStandardPaths::writableLocation(QStandardPaths::DesktopLocation);
    defaultPath += "/" + mChartName + " " + APP->getToday() + ".pdf";
    QString outputFile = QFileDialog::getSaveFileName(this, tr("Export PDF"), defaultPath,"*.pdf");
    if (outputFile.isEmpty()) {
        return;
    }

    QList<TChartWidget* > chartList = mChartMap.values();
    QPrinter printer;
    printer.setOutputFormat(QPrinter::PdfFormat);
    printer.setPageSize(pageTypeToEnum(iPageSize));
    printer.setOutputFileName(outputFile);

    int spacing = 20;
    int chartWidth = printer.width();
    int chartHeight = (printer.height() - spacing)/2;
    QPainter painter;
    painter.begin(&printer);
    int y = 0;
    for (TChartWidget* chart: chartList) {
        if (y + chartHeight > printer.height() ) {
            y = 0;
            printer.newPage();
        }
        QRect rect = QRect(0, y, chartWidth, chartHeight);
        chart->draw(painter, rect);
        y += chartHeight + spacing;
    }

    // UiLoader中的表格数据
    QList<TUiLoader*> uiloaderList = mUiLoaderMap.values();
    for (TUiLoader *uiloader: uiloaderList) {
        QList<TTableView*> tableviewList;
        QObjectList objectList = uiloader->getAllObject();
        for (QObject *obj: objectList) {
            if (TTableView *view = qobject_cast<TTableView*>(obj)) {
                tableviewList.append(view);
            }
        }
        for (TTableView *view: tableviewList) {
            QVariantList headerItems = view->headerItem();
            QVariantList dataList = view->allDataMap();
            QStringList field_text_list;
            QStringList field_name_list;
            for (QVariant var: headerItems) {
                QVariantMap headerMap = var.toMap();
                QString name = headerMap.value("name").toString();
                if (!name.isEmpty()) {
                    QString text = headerMap.value("display").toString();
                    QString realName = headerMap.value("displayRole").toString();
                    realName.remove("$");
                    field_name_list.append(realName);
                    field_text_list.append(text);
                }
            }
            QString html = QString("<table width='%1' border='1' align='center' frame='border' cellspacing='0' cellpadding='3px' rules='all'>").arg(printer.width());
            html += "<tr>";
            //表头字段
            for (QString ti: field_text_list) {
                html += QString("<th>%1</th>").arg(ti);
            }
            html += "</tr>";
            // 数据
            for (int i = 0; i < dataList.size(); ++i) {
                QVariantMap rowMap = dataList.at(i).toMap();
                html += "<tr>";
                for (QString name: field_name_list) {
                    QString text = rowMap.value(name).toString();
                    html += QString("<td>%1</td>").arg(text);
                }
                html += "</tr>";
            }
            html += "</table>";
            printer.newPage();
            QTextDocument textDoc;
            textDoc.setHtml(html);
            textDoc.drawContents(&painter);
            textDoc.end();
        }
    }
    painter.end();
    t::msgBoxExport(outputFile);
}

/*!
 * \brief 导出Excel
 */
void TopTemplateClass8::onExportExcelClicked()
{
    if (mChartMap.size() == 0 && mUiLoaderMap.size() == 0) {
        t::msgBoxError(tr("No chart to export"));
        return;
    }
    // Get save path
    QString defaultPath = QStandardPaths::writableLocation(QStandardPaths::DesktopLocation);
    defaultPath += "/" + mChartName + " " + APP->getToday() + ".xlsx";
    QString filePath = TFileDialog::getSaveFileName(this, tr("Export Excel"), defaultPath, "Excel Files (*.xlsx *.xls)");
    if (filePath.isEmpty()) {
        return;
    }
    // Init Book/Sheet
    Book *book = new Book;
    bool isOk = book->create(filePath);
    if (!isOk) {
        t::msgBoxError(tr("Create file failed!"));
        return;
    }
    QString sheetName = "Sheet1";
    Sheet *sheet = book->addSheet(sheetName);
    if (sheet->isNull()) {
        t::msgBoxError(tr("Add sheet failed!"));
        return;
    }
    this->setDisabled(true);
    int curRow = 0;
    // Chart Data
    QList<TChartWidget* > chartWidgetList = mChartMap.values();
    for (TChartWidget* chartWidget : chartWidgetList) {
        QString parameterRemark = mChartParameterRemarkMap.value(chartWidget);
        QVariantMap parameterRemarkMap = TDataParse::jsonStr2Variant(parameterRemark).toMap();
        QString title = chartWidget->title();
        TChartStore* chartStore = chartWidget->store();
        QStringList dataKeyList = chartStore->dataKeyList();
        QVariantList dataList = chartStore->dataList();
        QStringList field_text_list;
        for (QString dataKey : dataKeyList) {
            QString realDataKey = dataKey;
            QString parameterRemarkItem = parameterRemarkMap.value(dataKey).toString().trimmed();
            if (!parameterRemarkItem.isEmpty()) {
                realDataKey = parameterRemarkItem;
            }
            field_text_list.push_back(realDataKey);
        }
        // Title
        if (!title.isEmpty()) {
            Format *formatTitle = book->addFormat();
            Font *fontTitle = book->addFont();
            fontTitle->setName("Microsoft YaHei");
            fontTitle->setSize(15);
            fontTitle->setBold(true);
            formatTitle->setFont(fontTitle);
            formatTitle->setAlignH("CENTER");
            formatTitle->setAlignV("CENTER");
            formatTitle->setBorder("THIN");
            formatTitle->setBorderColor("#0070C0");
            sheet->setCellValue(curRow, 0, title, "", formatTitle);
            sheet->setMerge(curRow, curRow, 0, field_text_list.size() - 1);
            sheet->setRows(0, 0, 28);
            curRow++;
        }
        // Header
        Format *formatHeader = book->addFormat();
        Font *fontHeader = book->addFont();
        fontHeader->setName("Microsoft YaHei");
        fontHeader->setSize(12);
        fontHeader->setBold(true);
        formatHeader->setFont(fontHeader);
        formatHeader->setAlignH("CENTER");
        formatHeader->setAlignV("CENTER");
        formatHeader->setBorder("THIN");
        formatHeader->setBorderColor("#0070C0");
        for (int col = 0; col < field_text_list.size(); col++) {
            sheet->setCellValue(curRow, col, field_text_list.value(col), "", formatHeader);
        }
        curRow++;
        // Data
        Format *formatData = book->addFormat();
        Font *fontData = book->addFont();
        fontData->setName("Microsoft YaHei");
        fontData->setSize(12);
        formatData->setFont(fontData);
        formatData->setBorder("THIN");
        formatData->setBorderColor("#0070C0");
        for (int i = 0; i < dataList.size(); i++) {
            QVariantList rowDataList = dataList.value(i).toList();
            for (int col = 0; col < rowDataList.size(); col++) {
                QString val = rowDataList.value(col).toString().trimmed();
                sheet->setCellValue(curRow, col, val, "", formatData);
            }
            qDebug()<<"rowszie: "<<rowDataList.size();
            curRow++;
        }
        curRow++;
    }
    // UiLoader's table
    // 将表头解析成name列表，text列表
    std::function<void(const QVariantList&, QStringList &, QStringList &)> parseHeaderItems = [](const QVariantList &iHeaderItems,
            QStringList &iFieldNameList, QStringList &iFieldTextList) {
        for (QVariant var: iHeaderItems) {
            QVariantMap headerMap = var.toMap();
            QString name = headerMap.value("name").toString();
            if (!name.isEmpty()) {
                QString text = headerMap.value("display").toString();
                QString realName = headerMap.value("displayRole").toString();
                realName.remove("$");
                iFieldNameList.append(realName);
                iFieldTextList.append(text);
            }
        }
    };
    // 将树形的数据转为表格数据
    std::function<QVariantList(const QVariantList &)> extract = [&](const QVariantList &iDataList)
    {
        QVariantList retList;
        for (QVariant var : iDataList) {
            retList.append(var);
            QVariantList subList = var.toMap().value("CHILDREN").toList();
            if (!subList.isEmpty()) {
                retList.append(extract(subList));
            }
        }
        return retList;
    };
    QList<TUiLoader*> uiloaderList = mUiLoaderMap.values();
    for (TUiLoader *uiloader: uiloaderList) {
        QList<QAbstractItemView*> itemViewList;
        QObjectList objectList = uiloader->getAllObject();
        for (QObject *obj: objectList) {
            if (QAbstractItemView *view = qobject_cast<QAbstractItemView*>(obj)) {
                itemViewList.append(view);
            }
        }
        for (QAbstractItemView *view: itemViewList) {
            QVariantList dataList;
            QVariantList headerItems;
            QStringList field_text_list;
            QStringList field_name_list;

            // Header
            Format *formatHeader = book->addFormat();
            Font *fontHeader = book->addFont();
            fontHeader->setName("Microsoft YaHei");
            fontHeader->setSize(12);
            fontHeader->setBold(true);
            formatHeader->setFont(fontHeader);
            formatHeader->setAlignH("CENTER");
            formatHeader->setAlignV("CENTER");
            formatHeader->setBorder("THIN");
            formatHeader->setBorderColor("#0070C0");
            // TTableView 和 TTreeView表头配置不一样，因此代码分开处理
            if (TTableView *tableView = qobject_cast<TTableView*>(view)) {
                dataList = tableView->allDataMap();
                headerItems = tableView->headerItem();
                QVariantList rawHeaderItems = tableView->initialHeaderItem();
                // TTableView表头配置中第一列为列头，暂未使用
                headerItems.removeFirst();
                parseHeaderItems(headerItems, field_name_list, field_text_list);
                rawHeaderItems.removeFirst();
                int tmpCol = 0;
                int depth = calcTreeDepth(rawHeaderItems);
                curRow += depth;
                for (QVariant var: rawHeaderItems) {
                    curRow -= depth;
                    QVariantMap varMap = var.toMap();
                    sheet->setCellValue(curRow, tmpCol, varMap.value("display").toString(), "", formatHeader);
                    QVariantList children = varMap.value("children").toList();
                    int filledHeight = 0;
                    if (children.size() == 0) {
                        filledHeight = depth - 1;
                    }
                    int width = calcTreeLeafSize(varMap);
                    sheet->setMerge(curRow, curRow + filledHeight, tmpCol, tmpCol + width - 1);
                    int subRow = curRow;
                    while(1) {
                        QVariantList tmpChildren;
                        if (children.size() > 0) {
                            subRow++;
                            int subCol = tmpCol;
                            for (QVariant item: children) {
                                QVariantMap itemMap = item.toMap();
                                sheet->setCellValue(subRow, subCol, itemMap.value("display").toString(), "", formatHeader);
                                QVariantList tmpList = itemMap.value("children").toList();
                                int tmpWidth = calcTreeLeafSize(itemMap);
                                int tmpFilledHeight = 0;
                                if (tmpList.size() == 0) {
                                    tmpFilledHeight = curRow + depth - subRow - 1;
                                } else {
                                    tmpChildren.append(tmpList);
                                }
                                sheet->setMerge(subRow, subRow + tmpFilledHeight, subCol, subCol + tmpWidth - 1);
                                subCol += tmpWidth;
                            }
                        }
                        children = tmpChildren;
                        if (children.size() == 0) {
                            break;
                        }
                    }
                    tmpCol += width;
                    curRow += depth;
                }
            } else if (TTreeView *treeView = qobject_cast<TTreeView*>(view)) {
                dataList = extract(treeView->allDataMap());
                headerItems = treeView->headerItem();
                parseHeaderItems(headerItems, field_name_list, field_text_list);
                int tmpCol = 0;
                for (QString field : field_text_list) {
                    sheet->setCellValue(curRow, tmpCol, field, "", formatHeader);
                    tmpCol++;
                }
                curRow++;
            }

            // Data
            Format *formatData = book->addFormat();
            Font *fontData = book->addFont();
            fontData->setName("Microsoft YaHei");
            fontData->setSize(12);
            formatData->setFont(fontData);
            formatData->setBorder("THIN");
            formatData->setBorderColor("#0070C0");
            for (int i = 0; i < dataList.size(); i++) {
                QVariantMap rowMap = dataList.value(i).toMap();
                int col = 0;
                for (QString name: field_name_list) {
                    QString val = rowMap.value(name).toString().trimmed();
                    sheet->setCellValue(curRow, col++, val, "", formatData);
                }
                curRow++;
            }
            curRow++;
        }
    }
    sheet->setAutoFitArea();
    this->setDisabled(false);
    book->save();
    book->close();
    t::msgBoxExport(filePath);
}

/*!
 * \brief 设置不显示的图表
 */
void TopTemplateClass8::setChartVisibleData(const QVariantMap &iDataMap)
{
    mChartVisibleData = iDataMap;
}

/*!
 * \brief 不显示的图表
 */
QVariantMap TopTemplateClass8::chartVisibleData()
{
    return mChartVisibleData;
}

/*!
 * \brief 当选中发生改变时的槽函数
 */
void TopTemplateClass8::onSelectionChanged()
{
    if (isHookExists("onSelectionChanged")) {
        callHooks("onSelectionChanged");
    }
    refreshActionState();
}

/*!
 * \brief 当分页发生改变时的槽函数
 */
void TopTemplateClass8::onPageChanged()
{
    refresh(false);
}

/*!
 * \brief 当导航栏过滤选项发生改变时的槽函数
 */
void TopTemplateClass8::onCategoryViewDataChanged(const QVariantList iDataLst)
{
    mCategoryWhere.clear();
    QString format = config("navi.format", "filter_by_enum").toString();
    if (!iDataLst.isEmpty()) {
        if (format.compare("filter_by_enum") == 0) {
            QVariantMap cateMap;
            for (QVariant item: iDataLst) {
                QVariantMap itemMap = item.toMap();
                QString data = itemMap.value("data").toString();
                QString name = itemMap.value("name").toString();
                if (data.isEmpty()) {
                    continue;
                }
                cateMap.insertMulti(data,name);
            }
            for (QString field: cateMap.uniqueKeys()) {
                QString dbField = field;
                TSqlWhereCompsiteV2 childWhere;
                childWhere.setLogic(TSqlWhereCompsiteV2::Logic_Or);
                for (QVariant ele: cateMap.values(field)) {
                    QString value = ele.toString();
                    if (mTableConf.fieldFormatMap.contains(dbField)) {
                        QString fieldValue = mTableConf.fieldFormatMap.value(dbField).toString();
                        if (fieldValue.compare("array") == 0) {
                            QString sqlStr = QString("%1 @> '{%2}' ").arg(dbField).arg(value);
                            childWhere.append(sqlStr);
                        }
                    } else {
                        childWhere.append(dbField, value);
                    }
                }
                mCategoryWhere.append(childWhere);
            }
        } else if (format.compare("filter_by_sql_where") == 0) {
            QVariantMap item = iDataLst.at(0).toMap();
            QString sqlWhere = item.value("data").toString();
            if (!sqlWhere.isEmpty()) {
                mCategoryWhere.append(sqlWhere);
            }
        }
    }
    refresh();
}

/*!
 * \brief 当module中view配置改变时的槽函数
 */
void TopTemplateClass8::onViewConfChanged()
{
    TopClassHelper::parseTableConf0(this, "view", &mTableConf);
}

/**
* \brief 当高级查询界面的添加按钮触发时的槽函数
*/
void TopTemplateClass8::onAddActionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    TTableChooserDialog *dialog = new TTableChooserDialog(this);
    dialog->setTitle(tr("Please select field:"));
    dialog->setButtons(QStringList()
                       << ttr("Ok") + ":Ok:Yes"
                       << ttr("Cancel") + ":Cancel:Cancel"
                       << QString("%1:UnselectAll:ResetRole").arg(tr("Clear selection")));
    QVariantList headeritems;
    headeritems << QVariant() << TDataParse::stringList2VariantMap(
                       QStringList() << "name" << "title" << "display" << tr("Field") << "displayRole" << "$title"<<"resizeMode" << "Stretch");
    dialog->setHeaderItem(headeritems);
    dialog->setDataKeyList(QStringList() << "title" << "name");
    dialog->setPrimaryKey("name");
    dialog->setSearchKeys(QStringList() << "title");
    dialog->setLeftTableSelectionMode(QAbstractItemView::ExtendedSelection);
    dialog->setRightTableSelectionMode(QAbstractItemView::ExtendedSelection);
    dialog->tableView()->verticalHeader()->setVisible(false);
    dialog->selectedTableView()->verticalHeader()->setVisible(false);
    dialog->loadAllData(getAdvancedQueryFieldList());
    dialog->loadSelectedData(advancedQuery->getQueryData().value("value").toList());

    QVariantList ret = dialog->run();
    if (ret.isEmpty()) {
        return;
    }

    QStringList conditionLst;
    for (QVariant var : ret) {
        QString fieldName = var.toMap().value("name").toString();
        conditionLst << fieldName;
    }
    advancedQuery->clearSearchCondition();
    advancedQuery->addSearchCondition(conditionLst);
}

/**
* \brief 当高级查询界面的清除按钮触发时的槽函数
*/
void TopTemplateClass8::onClearActionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }
    advancedQuery->clearSearchCondition();
}

/**
* \brief 当高级查询界面的另存为按钮触发时的槽函数
*/
void TopTemplateClass8::onSaveAsActionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    QString fileName = QFileDialog::getSaveFileName(this,
                                                    tr("Save query criteria as file"),
                                                    QString(),
                                                    QString("*.cond"));
    if (fileName.isEmpty()) {
        return;
    }

    QVariantMap newConditionMap;
    newConditionMap.insert("condition", advancedQuery->getQueryData());
    QString conditionStr = TDataParse::variant2JsonStr(newConditionMap);
    TFileIo::writeFileString(conditionStr , fileName);
}

/**
* \brief 当高级查询界面的打开按钮触发时的槽函数
*/
void TopTemplateClass8::onOpenActtionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    QString fileName = QFileDialog::getOpenFileName(this,
                                                    tr("Open query criteria from file"),
                                                    ".",
                                                    tr("Condition file(*.cond)"));
    if (fileName.isEmpty()) {
        return;
    }
    QString conditionStr = TFileIo::readFileString(fileName);
    QVariantMap dataMap = TDataParse::jsonStr2Variant(conditionStr).toMap();
    QVariantMap conditionMap = dataMap.value("condition").toMap();
    advancedQuery->setData(conditionMap);
}

/**
* \brief 当高级查询界面的保存按钮触发时的槽函数
*/
void TopTemplateClass8::onSaveAsCommonQueryActionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    TTableView *commonQueryTable = findCommonQueryTable();
    if (commonQueryTable == nullptr) {
        return;
    }

    QString uiStr;
    uiStr = "function func(self) {"
                   " return [{"
                           " name: 'basic_formlayout',"
                            "type: 'FormLayout',"
                            "property:{"
                                    "label_alignment: 'Top | Right',"
                                    "horizontal_spacing: 20, vertical_spacing: 10, margin: 20"
                            "},"
                            "child: [{"
                                    "name: 'name',"
                                    "type: 'LineEdit',"
                                    "title: '" + tr("Query Name") + "',"
                                    "pack: {"
                                            "label: '" + tr("Query Name") + "'"
                                    "},"
                                    "validate: 'NOTNULL'"
                                    "}]"
                            "}]}";

    TUiloaderDialog *dialog = new TUiloaderDialog(this);
    dialog->setTitle(tr("Save as a common query"));
    dialog->setSelf(this);
    dialog->resize(250,100);
    dialog->setButtons(QStringList() << QString("%1:Ok:AcceptRole").arg(ttr("Ok")) << QString("%1:Cancel::RejectRole").arg(ttr("Cancel")));
    dialog->setUiStr(uiStr);

    QVariantMap ret = dialog->run(true);
    if (ret.isEmpty()) {
        return;
    }

    // 判断name是否已经存在
    QString conditionName = ret.value("name").toString();
    QVariantList conditionList = mAdvancedQueryTableDataMap.value("advance").toList();
    for (QVariant var: conditionList) {
        QVariantMap itemMap = var.toMap();
        if (conditionName == itemMap.value("title")) {
            TMessageBox::error(this, tr("Save as common condition"), tr("This name already exists!"));
            return;
        }
    }

    // 取新的查询条件
    QVariantMap conditionMap = advancedQuery->getQueryData();
    QVariantMap newConditionMap;
    newConditionMap.insert("title", conditionName);
    newConditionMap.insert("condition", conditionMap);
    conditionList << QVariant(newConditionMap);

    commonQueryTable->loadData(conditionList);
    mAdvancedQueryTableDataMap.insert("advance", conditionList);

    // 生成json格式数据
    QString newConditionStr = TDataParse::variant2JsonStr(mAdvancedQueryTableDataMap);
    TSqlInserterV2 inserter;
    inserter.setTable("sys_conf");
    QVariantMap saveMap;
    saveMap.insert("data", newConditionStr);
    saveMap.insert("path", QStringList() << this->moduleName() << "AdvancedQuery");
    inserter.setData(saveMap);
    inserter.setField(saveMap.uniqueKeys());
    inserter.setAutoIncrementField("id");
    inserter.uniqueFieldRef() << "path";

    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_REPLACE_ROW, QVariant::fromValue(inserter));
    TDataResponse dataRes(data.toMap());
    if (dataRes.hasError()) {
        alertError(ttr("Load data failed!"), dataRes.errText());
        return;
    }
}

/**
* \brief 当常用查询界面的表格双击条目时的槽函数
*/
void TopTemplateClass8::onCommonQueryTableDoubleClicked(const QModelIndex &index)
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    TTableView *commonQueryTable = findCommonQueryTable();
    if (commonQueryTable == nullptr) {
        return;
    }

    TTabWidget *advancedQueryTabWidget = findAdvancedQueryTabWidget();
    if (advancedQueryTabWidget == nullptr) {
        return;
    }

    advancedQuery->clearSearchCondition();
    TTableModel *tableModel = qobject_cast<TTableModel *>(commonQueryTable->model());
    QVariantMap rowDataMap = tableModel->rowDataMap(index);
    QVariantMap conditionMap = rowDataMap.value("condition").toMap();
    advancedQuery->setData(conditionMap);
    advancedQueryTabWidget->setCurrentIndex(0);
}

/**
* \brief 当常用查询界面的表格删除条目时的槽函数
*/
void TopTemplateClass8::onDeleleCommonQueryActtionTriggered()
{
    TTableView *commonQueryTable = findCommonQueryTable();
    if (commonQueryTable == nullptr) {
        return;
    }

    QVariantMap selectedConditionMap = commonQueryTable->currentRowDataMap().toMap();
    if (selectedConditionMap.isEmpty()) {
        return;
    }

    QString selectedTitle = selectedConditionMap.value("title").toString();
    QVariantList conditionList = mAdvancedQueryTableDataMap.value("advance").toList();;
    QVariantList conditionDataList;
    for (QVariant var : conditionList){
        QString title = var.toMap().value("title").toString();
        // 如果不是选中的项，表示需要保留
        if (title != selectedTitle) {
            conditionDataList << QVariant(var.toMap());
        }
    }
    commonQueryTable->loadData(conditionDataList);
    mAdvancedQueryTableDataMap.insert("advance", conditionDataList);

    //生成json格式数据
    QString newConditionStr = TDataParse::variant2JsonStr(mAdvancedQueryTableDataMap);
    TSqlInserterV2 inserter;
    inserter.setTable("sys_conf");
    QVariantMap saveMap;
    saveMap.insert("data", newConditionStr);
    saveMap.insert("path", QStringList() << this->moduleName() << "AdvancedQuery");
    inserter.setData(saveMap);
    inserter.setField(saveMap.uniqueKeys());
    inserter.setAutoIncrementField("id");
    inserter.uniqueFieldRef() << "path";
    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_REPLACE_ROW, QVariant::fromValue(inserter));
    TDataResponse dataRes(data.toMap());
    if (dataRes.hasError()) {
        alertError(ttr("Load data failed!"), dataRes.errText());
        return;
    }
}

void TopTemplateClass8::onNaviReset()
{
    TUiLoader *filterUi = nullptr;
    if (mNaviType == NaviType::UiLoaderAndCategoryTreeView) {
        filterUi = mNaviViewUi;
    } else {
        filterUi = qobject_cast<TUiLoader*>(mNaviView);
    }
    if (filterUi == nullptr) {
        return;
    }
    filterUi->clearValues();
    this->refresh();
}

/*!
 * \brief 初始化界面
 */
void TopTemplateClass8::initUi()
{
    mBodySplitter = new TSplitter(this);
    mBodySplitter->setObjectName("mBodySplitter");
    setCentralWidget(mBodySplitter);
    mMgtSplitter = new TSplitter(this);
    mBodySplitter->addWidget(mMgtSplitter);

    mBodyWgt = new QWidget(this);
    QVBoxLayout *bodyLayout = new QVBoxLayout(mBodyWgt);
    bodyLayout->setMargin(0);
    bodyLayout->setSpacing(0);
    mMgtSplitter->addWidget(mBodyWgt);
    initNaviView();

    mTableView = new TTableView(this);
    // 此ObjectName在其他逻辑中是有意义的，请勿删除
    mTableView->setObjectName("__view__");
    mTableView->setHeaderPopupEnabled(true);
    connect(mTableView->horizontalHeader(), SIGNAL(sortIndicatorChanged(int,Qt::SortOrder)),
            this, SLOT(refresh()));
    connect(mTableView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SLOT(onSelectionChanged()));
    connect(mTableView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SIGNAL(selectionChanged()));
    mTableView->setDataKeyList(mTableConf.dataKeys);
    mTableView->setPrimaryKey(mTableConf.primaryKey);
    QVariantList headerItems = mTableConf.horizontalHeaders;
    headerItems.prepend(QVariant());
    mTableView->setHeaderItem(headerItems);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("MAIN_TOOLBAR"))) {
        bodyLayout->addWidget(toolbar, 0);
        mSearchEntry = qobject_cast<TSearchEntry *>(uim()->getWidget("MAIN_TOOLBAR/SEARCH_ENTRY"));
        if (mSearchEntry != nullptr) {
            connect(mSearchEntry, SIGNAL(search(QString,QVariant)), this, SLOT(refresh()));
            mSearchEntry->setOptionList(TDataParse::headerItem2searchList(mTableView->headerItem()));
            mSearchEntry->setPlaceholderText(ttr("Search %1"));
        }
    }
    if (QMenu *tablePopup = qobject_cast<QMenu *>(uim()->getWidget("TABLEVIEW_POPUP"))) {
        mTableView->setContextMenu(tablePopup);
    }

    bodyLayout->addWidget(mTableView, 1);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("BOTTOM_TOOLBAR"))) {
        bodyLayout->addWidget(toolbar, 0);
        mPageTool = qobject_cast<TPageTool *>(uim()->getWidget("BOTTOM_TOOLBAR/PAGE_TOOL"));
        if (mPageTool != nullptr) {
            mPageTool->setPageSizeVisible(true);
            connect(mPageTool, SIGNAL(pageChanged(int,int)), this, SLOT(onPageChanged()));
        }
    }

    QWidget *detailWgt = new QWidget(this);
    QVBoxLayout *detailLayout = new QVBoxLayout(detailWgt);
    detailLayout->setSpacing(0);
    detailLayout->setMargin(0);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("DETAIL_TOOLBAR"))) {
        detailLayout->addWidget(toolbar, 0);
    }
    mScrollArea = new TScrollArea(this);
    mScrollArea->setContentsMargins(0, 0, 0, 0);
    detailLayout->addWidget(mScrollArea);
    mBodySplitter->addWidget(detailWgt);
    mBodySplitter->setStretchFactor(0, 1);
    mBodySplitter->setSizes(QList<int>{1, config("detail.perfect_width", 600).toInt()});
}

/*!
 * \brief 初始化导航栏
 */
void TopTemplateClass8::initNaviView()
{
    mNaviType = str2NaviType(config("navi.__type__", "CategoryTreeView").toString());
    if (mNaviType == NaviType::None) {
        return;
    }

    TAccordion *accordion = new TAccordion(this);
    // 此ObjectName在其他逻辑中是有意义的，请勿删除
    accordion->setObjectName("__navi__");
    accordion->setProperty("SS_BG","NAVI");
    accordion->setMinimumWidth(TTHEME_DP(config("navi.min_width", 150).toInt()));
    accordion->setMaximumWidth(TTHEME_DP(config("navi.max_width", 300).toInt()));
    accordion->setAccordionLocation(TAccordion::AccordionLocation::Left);
    QVBoxLayout *naviLayout = new QVBoxLayout(accordion);
    naviLayout->setMargin(0);
    naviLayout->setSpacing(0);
    QLabel *naviLabel = new QLabel(tr("Navigation"));
    naviLabel->setProperty("SS_TYPE", "SUBHEADER");
    naviLayout->addWidget(naviLabel,0);
    naviLayout->addSpacing(TTHEME_DP(4));
    accordion->expandButton()->setFixedHeight(TTHEME_DP(40));
    accordion->expandButton()->setToolTip(tr("Show Navigation"));
    accordion->collapseButton()->setToolTip(tr("Hide Navigation"));
    accordion->setIsExpanded(config("navi.is_expanded", false).toBool());
    mMgtSplitter->insertWidget(0,accordion);

    if (mNaviType == NaviType::UiLoader) {
        mNaviView = new TUiLoader(this);
        TUiLoader *filterUi = qobject_cast<TUiLoader*>(mNaviView);
        filterUi->setSelf(this);
        filterUi->setScriptEngine(APP->scriptEngine());
        filterUi->setUiStr(ui("navi").toString());

        QFrame *naviWgt = new QFrame(this);
        QVBoxLayout *leftLayout = new QVBoxLayout(naviWgt);
        leftLayout->addWidget(mNaviView);
        QHBoxLayout *leftBtnsLayout = new QHBoxLayout();
        leftBtnsLayout->setMargin(0);
        leftBtnsLayout->setSpacing(10);
        leftLayout->addLayout(leftBtnsLayout);
        QPushButton *resetBtn = new QPushButton(this);
        connect(resetBtn, SIGNAL(clicked(bool)), this, SLOT(onNaviReset()));
        leftBtnsLayout->addWidget(resetBtn);
        resetBtn->setText(tr("Reset"));
        resetBtn->setIcon(TRES->icon("rotate-right"));
        QPushButton *filterBtn = new QPushButton(this);
        connect(filterBtn, SIGNAL(clicked(bool)), this, SLOT(refresh()));
        leftBtnsLayout->addWidget(filterBtn);
        filterBtn->setText(tr("Filter"));
        filterBtn->setIcon(TRES->icon("filter"));
        leftLayout->addStretch(1);
        naviLayout->addWidget(naviWgt);
    } else if (mNaviType == NaviType::AdvancedQuery) {
        Q_ASSERT_X(false, "TopTemplateClass8", "This feature is not yet supported.");
    } else if (mNaviType == NaviType::NaviFilter) {
        Q_ASSERT_X(false, "TopTemplateClass8", "This feature is not yet supported.");
    } else if (mNaviType == NaviType::CategoryTreeViewAndAdvancedQuery) {
        // tab1-CategoryTreeView
        TCategoryTreeView *categoryTreeView = new TCategoryTreeView(this);
        categoryTreeView->setStyleSheet("TCategoryTreeView{border:NONE;background-color:transparent}");
        QVariantList rootChildList = parseCategoryConfig();
        if (rootChildList.isEmpty()) {
            mNaviView->setVisible(false);
            return;
        }
        categoryTreeView->setItemList(rootChildList);
        categoryTreeView->setCheckable(config("navi.is_checkable").toBool());
        connect(categoryTreeView, SIGNAL(selectionChanged(QVariantList)),
                this, SLOT(onCategoryViewDataChanged(QVariantList)));

        // tab2-tab1-ToolBar
        QToolBar *toolbar = new QToolBar(this);
        toolbar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        toolbar->setProperty("SS_SIZE", "SMALL");
        toolbar->setMovable(false);

        QAction *queryAction = toolbar->addAction(APP->getResIcon("search"), tr("Query"));
        connect(queryAction, SIGNAL(triggered()), this, SLOT(refresh()));
        toolbar->addSeparator();

        QAction* addAction = toolbar->addAction(APP->getResIcon("addto-circle-o"), tr("Add"));
        addAction->setToolTip(tr("Add query criteria"));
        connect(addAction, SIGNAL(triggered()), this, SLOT(onAddActionTriggered()));

        QAction* clearAction = toolbar->addAction(APP->getResIcon("eraser"), tr("Clear"));
        clearAction->setToolTip(tr("Clear all query criteria"));
        connect(clearAction, SIGNAL(triggered()), this, SLOT(onClearActionTriggered()));

        QAction* saveAsAction = toolbar->addAction(APP->getResIcon("save"), tr("Save As"));
        saveAsAction->setToolTip(tr("Save query criteria as file"));
        connect(saveAsAction, SIGNAL(triggered()), this, SLOT(onSaveAsActionTriggered()));

        QAction* openActtion = toolbar->addAction(APP->getResIcon("folder-open-o"), tr("Open"));
        openActtion->setToolTip(tr("Open query criteria from file"));
        connect(openActtion, SIGNAL(triggered()), this, SLOT(onOpenActtionTriggered()));

        QAction* saveAsCommonQueryAction = toolbar->addAction(APP->getResIcon("save-as"), tr("Save"));
        saveAsCommonQueryAction->setToolTip(tr("Save as a common query"));
        connect(saveAsCommonQueryAction, SIGNAL(triggered()), this, SLOT(onSaveAsCommonQueryActionTriggered()));

        // tab2-tab1-AdvancedQuery
        TAdvancedQuery *advancedQuery = new TAdvancedQuery(this);
        advancedQuery->setObjectName("AdvancedQuery");
        initAdvanceCfgMap();
        advancedQuery->setItemList(mAdvancedQueryCfgMap.value("advanced_items").toList());
        advancedQuery->setData(getDefaultQueryDataFromCfgMap());

        TScrollArea *scrollArea = new TScrollArea(this);
        scrollArea->setWidgetResizable(true);
        scrollArea->setWidget(advancedQuery);

        // tab2-tab1
        QMainWindow *mainWindow = new QMainWindow(this);
        mainWindow->addToolBar(toolbar);
        mainWindow->setCentralWidget(scrollArea);

        // tab2-tab2-TableView
        TTableView *commonQueryTable = new TTableView(this);
        commonQueryTable->setObjectName("CommonQueryTable");
        commonQueryTable->setShowGrid(false);
        commonQueryTable->setSelectionMode(QAbstractItemView::ExtendedSelection);
        commonQueryTable->verticalHeader()->setVisible(false);
        QVariantList headerItem;
        headerItem << QVariant() << TDataParse::variantList2Map(
                          QVariantList() <<"name" << "title" << "display" << tr("Query Name") << "resizeMode" << "Stretch" << "displayRole" << "$title");
        commonQueryTable->setHeaderItem(headerItem);
        commonQueryTable->setDataKeyList(QStringList() << "title" << "condition");
        commonQueryTable->setPrimaryKey("condition");
        connect(commonQueryTable, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(onCommonQueryTableDoubleClicked(const QModelIndex &)));

        QAction *deleleCommonQueryActtion = new QAction(APP->getResIcon("remove"), tr("Delete query"), this);
        connect(deleleCommonQueryActtion, SIGNAL(triggered()), this, SLOT(onDeleleCommonQueryActtionTriggered()));
        QMenu *contextMenu = new QMenu(this);
        contextMenu->addAction(deleleCommonQueryActtion);
        commonQueryTable->setContextMenu(contextMenu);

        // 刷新常用查询表格数据
        initCommonQueryMap();
        commonQueryTable->loadData(mAdvancedQueryTableDataMap.value("advance").toList());

        // 高级查询Tab
        TTabWidget *advancedQueryTabWidget = new TTabWidget(this);
        advancedQueryTabWidget->setObjectName("AdvancedQueryTabWidget");
        advancedQueryTabWidget->addTab(mainWindow, tr("Advanced Query"));
        advancedQueryTabWidget->addTab(commonQueryTable, tr("Common Query"));

        // 导航Tab
        mNaviView = new TTabWidget(this);
        TTabWidget *tabWidget = qobject_cast<TTabWidget *>(mNaviView);
        tabWidget->addTab(categoryTreeView, tr("Category"));
        tabWidget->addTab(advancedQueryTabWidget, tr("Advanced Query"));
        naviLayout->addWidget(mNaviView);
    } else if (mNaviType == NaviType::UiLoaderAndCategoryTreeView) {
        mNaviView = new QFrame(this);
        QVBoxLayout *leftLayout = new QVBoxLayout(mNaviView);
        // uiLoader
        mNaviViewUi = new TUiLoader(this);
        mNaviViewUi->setSelf(this);
        mNaviViewUi->setScriptEngine(APP->scriptEngine());
        mNaviViewUi->setUiStr(ui("navi").toString());
        leftLayout->addWidget(mNaviViewUi);
        QHBoxLayout *leftBtnsLayout = new QHBoxLayout();
        leftBtnsLayout->setMargin(0);
        leftBtnsLayout->setSpacing(10);
        leftLayout->addLayout(leftBtnsLayout);
        QPushButton *resetBtn = new QPushButton(this);
        connect(resetBtn, SIGNAL(clicked(bool)), this, SLOT(onNaviReset()));
        leftBtnsLayout->addWidget(resetBtn);
        resetBtn->setText(tr("Reset"));
        resetBtn->setIcon(TRES->icon("rotate-right"));
        QPushButton *filterBtn = new QPushButton(this);
        connect(filterBtn, SIGNAL(clicked(bool)), this, SLOT(refresh()));
        leftBtnsLayout->addWidget(filterBtn);
        filterBtn->setText(tr("Filter"));
        filterBtn->setIcon(TRES->icon("filter"));
        // categoryTreeView
        mNaviViewCat = new TCategoryTreeView(this);
        mNaviViewCat->setStyleSheet("TCategoryTreeView{border:NONE;background-color:transparent}");
        QVariantList rootChildList = parseCategoryConfig();
        if (rootChildList.isEmpty()) {
            mNaviViewCat->setVisible(false);
        }
        mNaviViewCat->setItemList(rootChildList);
        mNaviViewCat->setCheckable(config("navi.is_checkable").toBool());
        connect(mNaviViewCat, SIGNAL(selectionChanged(QVariantList)),
                this, SLOT(onCategoryViewDataChanged(QVariantList)));
        mNaviViewCat->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        leftLayout->addSpacing(6);
        leftLayout->addWidget(mNaviViewCat, 1);
        naviLayout->addWidget(mNaviView);
    } else {
        mNaviView = new TCategoryTreeView(this);
        mNaviView->setStyleSheet("TCategoryTreeView{border:NONE;background-color:transparent}");
        QVariantList rootChildList = parseCategoryConfig();
        if (rootChildList.isEmpty()) {
            mNaviView->setVisible(false);
            return;
        }
        TCategoryTreeView *categoryTreeView = qobject_cast<TCategoryTreeView *>(mNaviView);
        categoryTreeView->setItemList(rootChildList);
        categoryTreeView->setCheckable(config("navi.is_checkable").toBool());
        connect(categoryTreeView, SIGNAL(selectionChanged(QVariantList)),
                this, SLOT(onCategoryViewDataChanged(QVariantList)));
        naviLayout->addWidget(mNaviView);
    }
}

/**
* \brief 初始化高级查询配置数据
* mAdvancedQueryCfgMap结构如下：
* \code
    advance: {
      "advanced_items" : [
        {"name" : "attr_data.machine_code", "title" : "Machine Code" , "widgetType" : "LineEdit" , "wgt_prop" : "" , "optionList" : []},
        {"name" : "attr_data.machine_name", "title" : "Machine Name" , "widgetType" : "LineEdit" , "wgt_prop" : "" , "optionList" : []},
        {"name" : "title", "title" : "Title" , "widgetType" : "LineEdit" , "wgt_prop" : "" , "optionList" : []},
        {"name" : "class", "title" : "Class" , "widgetType" : "ComboBox" , "wgt_prop" : "" , "optionList" : "enum(tpm-machine-maintenance-plan-class)"}
      ],
      "file_suffix": ".conf"
      "condition" :{
        "condition": "1 and 2",
        "value": [
            {
                "ignoreEmpty": false,
                "name": "attr_data.machine_code",
                "operator": "LIKE",
                "title": "Machine Code",
                "valid": true,
                "value1": "00"
            },
            {
                "ignoreEmpty": false,
                "name": "attr_data.machine_name",
                "operator": "LIKE",
                "title": "Machine Name",
                "valid": true,
                "value1": "4"
            }
        ]
      }
    }
* \endcode
*/
void TopTemplateClass8::initAdvanceCfgMap()
{
    mAdvancedQueryCfgMap = config("advance").toMap();
    QVariantList queryItems = mAdvancedQueryCfgMap.value("advanced_items").toList();
    QVariantList newQueryItems;
    for (QVariant var : queryItems) {
        QVariantMap m = var.toMap();
        // 翻译title
        m.insert("title", ttr(m.value("title").toString()));
        // 转换枚举
        QVariantList optionList;
        QRegExp regExp("enum(.*)");
        if ((m.value("optionList").type() == QVariant::String) && (regExp.indexIn(m.value("optionList").toString()) != -1)) {
            if (regExp.capturedTexts().count() > 1) {
                optionList = TOPENM->enumList(regExp.capturedTexts().value(1).remove("(").remove(")"))->toComboList();
                m.insert("optionList", optionList);
            }
        }
        newQueryItems << m;
    }
    mAdvancedQueryCfgMap.insert("advanced_items", newQueryItems);
}

/**
* \brief 获取高级查询控件默认的查询条件
*/
QVariantMap TopTemplateClass8::getDefaultQueryDataFromCfgMap()
{
    QVariantMap conditionMap = config("advance").toMap().value("condition").toMap();
    QVariantList valueList = conditionMap.value("value").toList();
    for (QVariant &var: valueList) {
        QVariantMap m = var.toMap();
        m.insert("title", ttr(m.value("title").toString()));
        var = m;
    }
    conditionMap.insert("value", valueList);
    return conditionMap;
}

/**
* \brief 从高级查询配置中获取未使用条目的name和text，添加条目时使用
*/
QVariantList TopTemplateClass8::getAdvancedQueryFieldList()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return QVariantList();
    }

    QVariantList selectedFieldDataLst = advancedQuery->getQueryData().value("value").toList();
    QStringList selectedFieldLst;
    for (QVariant val : selectedFieldDataLst){
        selectedFieldLst.append(val.toMap().value("name").toString());
    }

    QVariantList queryItems = mAdvancedQueryCfgMap.value("advanced_items").toList();
    QVariantList fieldList;
    for (QVariant var : queryItems) {
        QVariantMap m = var.toMap();
        QVariantMap itemMap;
        if(!selectedFieldLst.contains(m.value("name").toString())) {
            itemMap.insert("name", m.value("name").toString());
            itemMap.insert("title", ttr(m.value("title").toString()));
            fieldList << QVariant(itemMap);
        }
    }
    return fieldList;
}

/**
* \brief 从sys_conf表查找出已保存的常用查询数据
* 数据结构如下：
* \code
{
  "advance": [
    {
      "condition": {
        "condition": "1 and 2",
        "value": [
          {
            "ignoreEmpty": false,
            "name": "attr_data.machine_code",
            "operator": "LIKE",
            "title": "设备编号",
            "valid": true,
            "value1": "00"
          },
          {
            "ignoreEmpty": false,
            "name": "attr_data.machine_name",
            "operator": "LIKE",
            "title": "设备名称",
            "valid": true,
            "value1": "4"
          }
        ]
      },
      "title": "1"
    }
  ]
}
* \endcode
*/
void TopTemplateClass8::initCommonQueryMap()
{
    TSqlSelectorV2 selector;
    selector.setTable("sys_conf");
    selector.setField(QStringList() << "data");
    selector.setWhere("path", QStringList() << this->moduleName() << "AdvancedQuery", "@>");
    TError err;
    QVariant data = runSqlQueryOnThreadSync(TOPSQLTHREAD_SELECT_MAP,
                                            QVariant::fromValue(selector),
                                            &err);
    if (err.isValid()) {
        alertError(ttr("Load data failed!"), err.text());
        return;
    }
    mAdvancedQueryTableDataMap = TDataParse::jsonStr2Variant(data.toMap().value("data").toString()).toMap();
}

/**
* \brief 根据objname从导航栏中找到高级查询控件
*/
TAdvancedQuery *TopTemplateClass8::findAdvancedQuery()
{
    return mNaviView->findChild<TAdvancedQuery *>("AdvancedQuery");
}

/**
* \brief 根据objname从导航栏中找到常用查询tableView控件
*/
TTableView *TopTemplateClass8::findCommonQueryTable()
{
    return mNaviView->findChild<TTableView *>("CommonQueryTable");
}

/**
* \brief 根据objname从导航栏中找到高级查询TabWidget控件
*/
TTabWidget *TopTemplateClass8::findAdvancedQueryTabWidget()
{
    return mNaviView->findChild<TTabWidget *>("AdvancedQueryTabWidget");
}

/*!
 * \brief 解析导航栏配置
 */
QVariantList TopTemplateClass8::parseCategoryConfig()
{
    QVariantList categoryLst;

    QVariantList categoryConfig = config("navi.categories").toList();
    QString format = config("navi.format", "filter_by_enum").toString();
    if (format.compare("filter_by_enum") == 0) {
        for(const QVariant &item: categoryConfig) {
            QVariantMap itemMap = item.toMap();
            QString itemName         = itemMap.value("name").toString();
            QVariantList childrenLst = itemMap.value("children").toList();
            QString enumCfg          = itemMap.value("enum_children").toString();
            QStringList enumCheckedLst = itemMap.value("enum_checked").toStringList();
            QStringList enumInvisibleLst = itemMap.value("enum_invisible").toStringList();
            if (itemMap.contains("visible")) {
                itemMap.insert("VISIBLE", itemMap.take("visible"));
            }
            if (itemMap.contains("expand")) {
                itemMap.insert("EXPAND", itemMap.take("expand"));
            }
            TSqlWhereCompsiteV2 childWhere;
            childWhere.setLogic(TSqlWhereCompsiteV2::Logic_Or);
            QVariantList type = TOPENM->enumList(enumCfg)->toComboList();
            for (const QVariant &item: type) {
                QVariantMap enumMap = item.toMap();
                QString enumName = enumMap.value("name").toString();
                QVariantMap childMap;
                childMap.insert("name",enumName);
                childMap.insert("text",ttr(enumMap.value("text").toString()));
                childMap.insert("icon",enumMap.value("icon"));
                childMap.insert("data", itemName);
                if(enumCheckedLst.contains(enumName)) {
                    childMap.insert("checked",1);
                } else {
                    childMap.insert("checked",0);
                }
                if (enumInvisibleLst.contains(enumName)) {
                    childMap.insert("VISIBLE", 0);
                } else {
                    childMap.insert("VISIBLE",1);
                }
                childrenLst.append(childMap);
            }
            int checkedChildLen = 0;
            for (QVariant &item : childrenLst) {
                QVariantMap itemMap = item.toMap();
                QString name = itemMap.value("name").toString();
                if(itemMap.value("checked").toInt() == 1) {
                    childWhere.append(itemName, name);
                    checkedChildLen++;
                }
                item = itemMap;
            }
            if (checkedChildLen == childrenLst.length()) {
                itemMap.insert("checked", 1);
            } else {
                itemMap.insert("checked", 0);
            }
            mCategoryWhere.append(childWhere);

            itemMap.remove("children");
            itemMap.remove("enum_children");
            itemMap.remove("enum_checked");
            itemMap.remove("enum_invisible");
            itemMap.insert("text",ttr(itemMap.value("text").toString()));
            itemMap.insert("CHILDREN",childrenLst);

            categoryLst.append(itemMap);
        }
    } else if (format.compare("filter_by_sql_where") == 0) {
        return categoryConfig;
    }
    return categoryLst;
}

TopTemplateClass8::NaviType TopTemplateClass8::str2NaviType(const QString &iType)
{
    const QString type = iType.toUpper();
    if (type == "UILOADER") {
        return NaviType::UiLoader;
    } else if (type == "ADVANCEDQUERY") {
        return NaviType::AdvancedQuery;
    } else if (type == "NAVIFILTER") {
        return NaviType::NaviFilter;
    } else if (type == "CATEGORYTREEVIEWANDADVANCEDQUERY") {
        return NaviType::CategoryTreeViewAndAdvancedQuery;
    } else if (type == "UILOADERANDCATEGORYTREEVIEW") {
        return NaviType::UiLoaderAndCategoryTreeView;
    } else if (type == "NONE") {
        return NaviType::None;
    } else {
        return NaviType::CategoryTreeView;
    }
}

QString TopTemplateClass8::uiloaderWhere() const
{
    TUiLoader *filterUi = nullptr;
    if (mNaviType == NaviType::UiLoaderAndCategoryTreeView) {
        filterUi = mNaviViewUi;
    } else {
        filterUi = qobject_cast<TUiLoader*>(mNaviView);
    }
    if (filterUi == nullptr) {
        return QString();
    }

    auto filterDataMap = filterUi->getAllValues().toVariant().toMap();

    QString searchSql;
    QMapIterator<QString, QVariant> iter(filterDataMap);
    while (iter.hasNext()) {
        iter.next();
        QString key = iter.key();
        QString value = iter.value().toString();
        if (value.isEmpty()) {
            continue;
        }
        auto obj = filterUi->getObject(key);
        if (obj != nullptr) {
            QVariantMap userData = obj->property("tui_user_data").toMap();
            if (userData.contains("field_name")) {
                key = userData.value("field_name").toString();
            }
            QString operatorStr = "=";
            if (userData.contains("operator")) {
                operatorStr = userData.value("operator").toString();
            }
            if (operatorStr.compare("like", Qt::CaseInsensitive) == 0 ||
                    operatorStr.compare("ilike", Qt::CaseInsensitive) == 0) {
                value = QString("'%%1%'").arg(value);
            } else if (operatorStr.compare("in", Qt::CaseInsensitive) == 0) {
                value = QString("(%1)").arg(value);
            } else {
                if (iter.value().type() == QVariant::String) {
                    value = QString("'%1'").arg(value);
                }
            }
            QString searchStr = QString("%1 %2 %3").arg(key).arg(operatorStr).arg(value);
            if (!searchSql.isEmpty()) {
                searchSql.append(" and ");
            }
            searchSql.append(searchStr);
        }
    }
    return searchSql;
}

void TopTemplateClass8::dynamicGenerateChart(const int &pId, const QString &funcJsonData, const QVariantMap &packMap, const QString &wgtType)
{
    // 1得到数据源函数
    TSqlSelectorV2 selector;
    selector.setTable("pub_chart_graph AS PCG INNER JOIN pub_chart_datasource AS PCD ON PCG.chart_datasource_id = PCD.id");
    selector.setField("PCG.name AS graph_name,PCG.data AS graph_data,PCG.parameter_remark,PCD.data AS datasource_data");
    selector.setWhere(QString("PCG.id = %1").arg(pId));
    QVariant dataRet = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_MAP, QVariant::fromValue(selector));
    TDataResponse dataRes(dataRet.toMap());
    if (dataRes.hasError()) {
        alertError(dataRes.errText(), "Error");
        return;
    }
    QVariantMap data = dataRes.data().toMap();
    QString graphName = data.value("graph_name").toString();
    QString datasourceData = data.value("datasource_data").toString();
    QString graphData = data.value("graph_data").toString();
    QString graphParameterRemark = data.value("parameter_remark").toString();

    QScriptValue datasourceFun = APP->scriptEngine()->evaluate(QString("(%1)").arg(datasourceData));
    QScriptValueList datasourceFunArgs;
    datasourceFunArgs << funcJsonData;

    // 2得到数据源执行的结果,是个JSON
    QScriptValue datasourceFunRes = datasourceFun.call(APP->scriptEngine()->newQObject(this), datasourceFunArgs);
    QString datasourceFunResStr = datasourceFunRes.toString();
    TError error;
    QVariantMap datasourceResDataMap = TDataParse::jsonStr2Variant(datasourceFunResStr, &error).toMap();
    if (error.isValid()) {
        qDebug() << "error line at " << __LINE__;
        TMessageBox::error(this, ttr("Error"), error.text());
        return;
    }
    // 如果返回的状态是Error就不画了
    if (datasourceResDataMap.value("status").toString() == "Error") {
        if (datasourceResDataMap.value("message").isValid()) {
            TMessageBox::error(this, ttr("Error"), datasourceResDataMap.value("message").toString());
            qDebug() << "show error message";
        }
        qDebug() << "status is Error";
        return;
    }

    // 3执行动态生成配置文件的脚本
    QScriptValue graphFun = APP->scriptEngine()->evaluate(QString("(%1)").arg(graphData));
    QScriptValueList graphFunArgs;
    graphFunArgs << datasourceFunResStr;
    QScriptValue graphFunRes = graphFun.call(APP->scriptEngine()->newQObject(this), graphFunArgs);
    QString chartCfgStr = graphFunRes.toString();

    // 4生成Chart
    TChartWidget *chartWidget = new TChartWidget(mChartListWgt);
    chartWidget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    connect(chartWidget, SIGNAL(doubleClicked(QString,double)), this, SLOT(onDoubleClicked()));

    TChartStore *chartStore = new TChartStore(chartWidget);
    QVariantList keyVaraintList = datasourceResDataMap.value("keyList").toList();
    QStringList dataKeyList;
    for (QVariant item : keyVaraintList) {
        dataKeyList.append(item.toString());
    }
    chartStore->setDataKeyList(dataKeyList);
    chartStore->setDataList(datasourceResDataMap.value("dataList").toList());
    chartWidget->setStore(chartStore);
    chartWidget->setConfig(TDataParse::jsonStr2Variant(chartCfgStr, &error).toMap());
    if (error.isValid()) {
        qDebug() << "error line at " << __LINE__;
        TMessageBox::error(this, ttr("Error"), error.text());
        return;
    }
    if (wgtType == "gridlayout") {
        if (TGridLayout* layout = qobject_cast<TGridLayout*>(mChartListWgt->layout())) {
            int fromRow = packMap.value("fromRow").toInt();
            int fromColumn = packMap.value("fromColumn").toInt();
            int rowSpan = packMap.value("rowSpan").toInt();
            int columnSpan = packMap.value("columnSpan").toInt();
            layout->addWidget(chartWidget, fromRow, fromColumn, rowSpan, columnSpan);
            // 如果存在该Chart的话就要把它移除掉
            if (mChartMap.contains(graphName)) {
                TChartWidget* chartPoint = mChartMap.value(graphName);
                if (chartPoint) {
                    chartPoint->deleteLater();
                    chartPoint = NULL;
                    mChartMap.insert(graphName, chartWidget);
                    mChartParameterRemarkMap.insert(chartWidget, graphParameterRemark);
                }
            } else {
                mChartMap.insert(graphName, chartWidget);
                mChartParameterRemarkMap.insert(chartWidget, graphParameterRemark);
            }
            if (!mChartVisibleData.value(graphName, true).toBool()) {
                layout->removeWidget(chartWidget);
                chartWidget->setVisible(false);
            }
        }
    }
}

void TopTemplateClass8::configGenerateChart(const int &pId, const QString &funcJsonData, const QVariantMap &packMap, const QString &wgtType)
{
    // 1得到数据源函数
    TSqlSelectorV2 selector;
    selector.setTable("pub_chart_graph AS PCG INNER JOIN pub_chart_datasource AS PCD ON PCG.chart_datasource_id = PCD.id");
    selector.setField("PCG.name AS graph_name,PCG.data AS graph_data,PCG.parameter_remark,PCD.data AS datasource_data");
    selector.setWhere(QString("PCG.id = %1").arg(pId));
    QVariant dataRet = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_MAP, QVariant::fromValue(selector));
    TDataResponse dataRes(dataRet.toMap());
    if (dataRes.hasError()) {
        alertError(dataRes.errText(), "Error");
        return;
    }
    QVariantMap data = dataRes.data().toMap();
    QString graphName = data.value("graph_name").toString();
    QString datasourceData = data.value("datasource_data").toString();
    QString graphData = data.value("graph_data").toString();
    QString graphParameterRemark = data.value("parameter_remark").toString();

    QScriptValue datasourceFun = APP->scriptEngine()->evaluate(QString("(%1)").arg(datasourceData));
    QScriptValueList datasourceFunArgs;
    datasourceFunArgs << funcJsonData;

    // 2得到数据源执行的结果,是个JSON
    QScriptValue datasourceFunRes = datasourceFun.call(APP->scriptEngine()->newQObject(this), datasourceFunArgs);
    QString datasourceFunResStr = datasourceFunRes.toString();
    TError error;
    QVariantMap datasourceResDataMap = TDataParse::jsonStr2Variant(datasourceFunResStr, &error).toMap();
    if (error.isValid()) {
        qDebug() << "error line at " << __LINE__;
        TMessageBox::error(this, ttr("Error"), error.text());
        return;
    }
    // 如果返回的状态是Error就不画了
    if (datasourceResDataMap.value("status").toString() == "Error") {
        qDebug() << "status is Error";
        return;
    }

    QString chartCfgStr = graphData;
    // 4生成Chart
    TChartWidget *chartWidget = new TChartWidget(mChartListWgt);
    chartWidget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    TChartStore *chartStore = new TChartStore(chartWidget);
    QVariantList keyVaraintList = datasourceResDataMap.value("keyList").toList();
    QStringList dataKeyList;
    for (QVariant item : keyVaraintList) {
        dataKeyList.append(item.toString());
    }
    chartStore->setDataKeyList(dataKeyList);
    chartStore->setDataList(datasourceResDataMap.value("dataList").toList());
    chartWidget->setStore(chartStore);
    chartWidget->setConfig(TDataParse::jsonStr2Variant(chartCfgStr, &error).toMap());
    if (error.isValid()) {
        qDebug() << "error line at " << __LINE__;
        TMessageBox::error(this, ttr("Error"), error.text());
        return;
    }
    if (wgtType == "gridlayout") {
        if (TGridLayout* layout = qobject_cast<TGridLayout*>(mChartListWgt->layout())) {
            int fromRow = packMap.value("fromRow").toInt();
            int fromColumn = packMap.value("fromColumn").toInt();
            int rowSpan = packMap.value("rowSpan").toInt();
            int columnSpan = packMap.value("columnSpan").toInt();
            layout->addWidget(chartWidget, fromRow, fromColumn, rowSpan, columnSpan);
            // 如果存在该Chart的话就要把它移除掉
            if (mChartMap.contains(graphName)) {
                TChartWidget* chartPoint = mChartMap.value(graphName);
                if (chartPoint) {
                    chartPoint->deleteLater();
                    chartPoint = NULL;
                    mChartMap.insert(graphName, chartWidget);
                    mChartParameterRemarkMap.insert(chartWidget, graphParameterRemark);
                }
            } else {
                mChartMap.insert(graphName, chartWidget);
                mChartParameterRemarkMap.insert(chartWidget, graphParameterRemark);
            }
            if (!mChartVisibleData.value(graphName, true).toBool()) {
                layout->removeWidget(chartWidget);
                chartWidget->setVisible(false);
            }
        }
    }
}

void TopTemplateClass8::generateUiLoader(const int &pId, const QString &funcJsonData, const QVariantMap &packMap, const QString &wgtType)
{
    // 1得到数据源函数
    TSqlSelectorV2 selector;
    selector.setTable("pub_chart_graph AS PCG INNER JOIN pub_chart_datasource AS PCD ON PCG.chart_datasource_id = PCD.id");
    selector.setField("PCG.name AS graph_name,PCG.data AS graph_data,PCG.parameter_remark,PCD.data AS datasource_data");
    selector.setWhere(QString("PCG.id = %1").arg(pId));
    QVariant dataRet = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_MAP, QVariant::fromValue(selector));
    TDataResponse dataRes(dataRet.toMap());
    if (dataRes.hasError()) {
        alertError(dataRes.errText(), "Error");
        return;
    }
    QVariantMap data = dataRes.data().toMap();
    QString graphName = data.value("graph_name").toString();
    QString datasourceData = data.value("datasource_data").toString();
    QString graphData = data.value("graph_data").toString();
    QString graphParameterRemark = data.value("parameter_remark").toString();
    // 生成uiloader
    TUiLoader *uiLoader = new TUiLoader(mChartListWgt);
    uiLoader->setScriptEngine(APP->scriptEngine());
    uiLoader->setSelf(this);
    uiLoader->setUiStr(graphData);

    QScriptValue datasourceFun = APP->scriptEngine()->evaluate(QString("(%1)").arg(datasourceData));
    QScriptValueList datasourceFunArgs;
    datasourceFunArgs << funcJsonData << APP->scriptEngine()->newQObject(uiLoader);

    // 2得到数据源执行的结果,是个JSON
    QScriptValue datasourceFunRes = datasourceFun.call(APP->scriptEngine()->newQObject(this), datasourceFunArgs);
    QString datasourceFunResStr = datasourceFunRes.toString();
    TError error;
    QVariantMap datasourceResDataMap = TDataParse::jsonStr2Variant(datasourceFunResStr, &error).toMap();
    if (error.isValid()) {
        qDebug() << "error line at " << __LINE__;
        TMessageBox::error(this, ttr("Error"), error.text());
        return;
    }
    // 如果返回的状态是Error就不画了
    if (datasourceResDataMap.value("status").toString() == "Error") {
        if (datasourceResDataMap.value("message").isValid()) {
            TMessageBox::error(this, ttr("Error"), datasourceResDataMap.value("message").toString());
            qDebug() << "show error message";
        }
        qDebug() << "status is Error";
        return;
    }

    QVariantMap uiDataMap = TDataParse::jsonStr2Variant(datasourceFunResStr, &error).toMap();
    if (error.isValid()) {
        qDebug() << "error line at " << __LINE__;
        TMessageBox::error(this, ttr("Error"), error.text());
        return;
    }
    uiLoader->loadValues(uiDataMap);

    if (wgtType == "gridlayout") {
        if (TGridLayout* layout = qobject_cast<TGridLayout*>(mChartListWgt->layout())) {
            int fromRow = packMap.value("fromRow").toInt();
            int fromColumn = packMap.value("fromColumn").toInt();
            int rowSpan = packMap.value("rowSpan").toInt();
            int columnSpan = packMap.value("columnSpan").toInt();
            layout->addWidget(uiLoader, fromRow, fromColumn, rowSpan, columnSpan);
            // 如果存在该UiLoader的话就要把它移除掉
            if (mUiLoaderMap.contains(graphName)) {
                TUiLoader *tmpLoader = mUiLoaderMap.value(graphName);
                if (tmpLoader) {
                    tmpLoader->deleteLater();
                    tmpLoader = nullptr;
                    mUiLoaderMap.insert(graphName, uiLoader);
                    mUiLoaderParamRemarkMap.insert(uiLoader, graphParameterRemark);
                }
            } else {
                mUiLoaderMap.insert(graphName, uiLoader);
                mUiLoaderParamRemarkMap.insert(uiLoader, graphParameterRemark);
            }
            if (!mChartVisibleData.value(graphName, true).toBool()) {
                layout->removeWidget(uiLoader);
                uiLoader->setVisible(false);
            }
        }
    }
}

QVariantList TopTemplateClass8::listPageType()
{
    QVariantList list;
    QStringList pageTypeList;
    pageTypeList << "A0" << "A1" << "A2" << "A3" << "A4" << "A5" << "A6"
                 << "A7" << "A8" << "A9" << "B0" << "B1" << "B2" << "B3"
                 << "B4" << "B5" << "B6" << "B7" << "B8" << "B9" << "B10";
    for (QString pageType: pageTypeList) {
        QVariantMap pageTypeMap;
        pageTypeMap.insert("name", pageType);
        pageTypeMap.insert("text", pageType);
        list.append(pageTypeMap);
    }
    return list;
}

QPagedPaintDevice::PageSize TopTemplateClass8::pageTypeToEnum(const QString &pageSize)
{
    if (pageSize == "A0") {
        return QPrinter::A0;
    } else if (pageSize == "A1") {
        return QPrinter::A1;
    } else if (pageSize == "A2") {
        return QPrinter::A2;
    } else if (pageSize == "A3") {
        return QPrinter::A3;
    } else if (pageSize == "A4") {
        return QPrinter::A4;
    } else if (pageSize == "A5") {
        return QPrinter::A5;
    } else if (pageSize == "A6") {
        return QPrinter::A6;
    } else if (pageSize == "A7") {
        return QPrinter::A7;
    } else if (pageSize == "A8") {
        return QPrinter::A8;
    } else if (pageSize == "A9") {
        return QPrinter::A9;
    } else if (pageSize == "B0") {
        return QPrinter::B0;
    } else if (pageSize == "B1") {
        return QPrinter::B1;
    } else if (pageSize == "B2") {
        return QPrinter::B2;
    } else if (pageSize == "B3") {
        return QPrinter::B3;
    } else if (pageSize == "B4") {
        return QPrinter::B4;
    } else if (pageSize == "B5") {
        return QPrinter::B5;
    } else if (pageSize == "B6") {
        return QPrinter::B6;
    } else if (pageSize == "B7") {
        return QPrinter::B7;
    } else if (pageSize == "B8") {
        return QPrinter::B8;
    } else if (pageSize == "B9") {
        return QPrinter::B9;
    } else if (pageSize == "B10") {
        return QPrinter::B10;
    } else {
        return QPrinter::Custom;
    }
}

int TopTemplateClass8::calcTreeDepth(const QVariantList &iDataList)
{
    int maxDepth = 0;
    for (QVariant var: iDataList) {
        QVariantMap varMap = var.toMap();
        QVariantList children = varMap.value("children").toList();
        if (children.size() > 0) {
            int tmpDepth = calcTreeDepth(children);
            if (tmpDepth > maxDepth) {
                maxDepth = tmpDepth;
            }
        }
    }
    return maxDepth + 1;
}

int TopTemplateClass8::calcTreeDepth(const QVariantMap &iDataMap)
{
    int depth = 0;
    QVariantList children = iDataMap.value("children").toList();
    if (children.size() == 0) {
        depth = 1;
    } else {
        depth = calcTreeDepth(children);
    }
    return depth;
}

int TopTemplateClass8::calcTreeLeafSize(const QVariantMap &iDataMap)
{
    int leafSize = 0;
    QVariantList children = iDataMap.value("children").toList();
    if (children.size() == 0) {
        leafSize++;
    } else {
        for (QVariant var: children) {
            QVariantMap varMap = var.toMap();
            leafSize += calcTreeLeafSize(varMap);
        }
    }
    return leafSize;
}
